class DevicesController < RxModule::ApplicationController
  model_data_store(Device)

  include IsComponent

  def before_set_targets
    Device.joins(:component) #sub with device classname and the name of the component relationship
  end

  def create
    params = model_params
    if Setting.first[:connection_type] == 'OPCUA'
      component = OpcuaComponent.new(name: params[:name])
      device = OpcuaDevice.new
      component.device = device
      device_type = DeviceType.find_by(id: params[:device_type_id])
      if params[:routing_retry_delay]
        device[:routing_retry_delay] = params[:routing_retry_delay]
      else
        device[:routing_retry_delay] = device_type[:routing_retry_delay]
      end
      if params[:delay_on_command_complete]
        device[:delay_on_command_complete] = params[:delay_on_command_complete]
      else
        device[:delay_on_command_complete] = device_type[:delay_on_command_complete]
      end

      device_type.opcua_devices << device

      device_type.get_unique_opcua_connectors.each do |connector|
        connector = OpcuaConnector.new({
            name: connector[:name],
            variable: connector[:variable],
            connector_type: connector[:connector_type],
            priority: connector[:priority],
            action_delay: connector[:action_delay],
            state: 'online'
        })
        component.connectors << connector
      end

      device_type.device_timers.each do |timer|
        timer = DeviceTimer.new({
            variable: timer[:variable],
            time: timer[:time],
            description: timer[:description],
        })
        component.device_timers << timer
      end

      drci = device_type.device_routing_command_ids.map {|drci| {device_routing_command_id: drci}}
      device.device_routing_command_devices.create(drci)
    else
      component = MesComponent.new(name: params[:name])
      component.connectors.new(name: 'input', connector_type: 'input', path: 0, type: 'MesConnector')
      for i in 0..(params[:number_of_outputs].to_i - 1) do
        component.connectors.new(name: 'output-'+i.to_s, connector_type: 'output', path: i, type: 'MesConnector')
      end
      device = MesDevice.new
      component.device = device
    end

    if component.save #&& device_type.save
      render json: device
    else
      render json: component.errors, status: :unprocessable_entity
    end
  end

  def update
    params = model_params
    component = @model.component

    component[:name] = params[:name] if component[:name] != params[:name]
    @model[:routing_retry_delay] = params[:routing_retry_delay]
    @model[:delay_on_command_complete] = params[:delay_on_command_complete]
    @model[:send_job_state_as] = params[:send_job_state_as]
    @model[:job_state_process] = params[:job_state_process]

    if @model.save && component.save
      JobRouter::Client.device_update(@model)
      render :show
    else
      render json: component.errors, status: :unprocessable_entity
    end
  end

  def destroy
    if @model.destroy
      routing_graph = RoutingGraph.first
      routing_graph.remove_node(@model.component[:id])
      render json: @model
    else
      render json: @model.errors, status: :unprocessable_entity
    end
  end

  def update_state
    params = model_params
    device = Device.find(params[:id])
    io_connectors = device.component.connectors.where.not(connector_type: "other")
    io_connectors.update_all(state: params[:state])
    device.component.transaction do
      io_connectors.each do |connector|
        connector.save
      end
    end

    JobRouter::Client.connectors_update(io_connectors)
    json = device.as_json
    json[:state] = device.device_state
    json[:name] = device.component[:name]
    json[:num_scanners] = device.component.connectors.where.not(rx_scanner_reference: nil).count
    ActionCable.server.broadcast "everything", {
        model: JSON.parse(device.render_jbuilder_partial),
        class: 'Device',
        action: :update
    }
    render json: json
  end

  def force_reset
    device = Device.find(params[:device_id])
    JobRouter::Client.device_force_reset(device)
    render json: {status: "device #{device.component[:name]} successfully reset"}
  end

  def to_json(*args)
    render_jbuilder_partial
  end

  private

  def model_params
    params.fetch(:device).permit( :id, :name, :device_type_id, :routing_retry_delay, :delay_on_command_complete, :state, :number_of_outputs, :send_job_state_as, :job_state_process)
  end
end

