class JobsController < RxModule::ApplicationController
  include PushDataResponse

  include RouterRoutes::Jobs

  def self.set_target_before
    super + [:update_required_processes, :finish, :reset, :job_history]
  end

  model_data_store(Job)

  also_protect_with([RxModule::Authorization.verb("read", "Read", :search)])

  def index
    job_numbers = targets_external_id
    begin
      if Setting.first.connection_type == 'MES'
        @rx_data_job_status = RxModule::Communicator.call_module_method('rx_data', '/api/jobs_status', :get, {job_numbers: job_numbers}).stringify_keys
      else
        @rx_data_job_status = {}
      end
    rescue => e
      @rx_data_job_status = {}
    end
    render :index
  end

  def show
    job_numbers = [{
         job_number: @model.number,
         rx_data_job_id: @model.rx_data_job_id
     }]
    begin
      if Setting.first.connection_type == 'MES'
        @rx_data_job_status = RxModule::Communicator.call_module_method('rx_data', '/api/jobs_status', :get, {job_numbers: job_numbers}).stringify_keys
      else
        @rx_data_job_status = {}
      end
    rescue => e
      @rx_data_job_status = {}
    end
    render :show
  end

  def create
    job_number = create_params[:job_number]
    begin
      @model = JobRequest.request_new(job_number)
      @model.update(is_routing: true)
      JobRouter::Client.job_create(@model)
    rescue => e
      Rails::logger.warn "Job Create failed with error: #{e}"
      render json: {error: e}, status: :unprocessable_entity
    end

    render 'jobs/show'
  end

  def search
    job = Job.find_by_number(params[:job_number])
    if job.nil?
      render json: {status: 'failed'}
    else
      render json: {id: job[:id], status: 'success'}
    end
  end

  def destroy
    job = Job.find(params[:id])
    job.do_delete
  end

  def fetch_job_status
    job = Job.find_by(id: params[:job_id])
    job_numbers = [{
         job_number: job.number,
         rx_data_job_id: job.rx_data_job_id
     }]
    begin
      rx_data_job_status = RxModule::Communicator.call_module_method('rx_data', '/api/jobs_status', :get, {job_numbers: job_numbers}).stringify_keys
    rescue => e
      rx_data_job_status = {}
    end
    render json: {status: rx_data_job_status[job.number]}
  end

  def finish
    @model.finish
    render json: {status: 'Job finished'}
  end

  def reset
    @model.reset
    render json: {status: 'Job reset'}
  end

  def reject
    job = Job.find_by(id: params[:job_id])
    if job.nil?
      Rails::logger.warn "Job with id #{params[:job_id]} not found"
      return render json: {status: 0}, status: :unprocessable_entity
    end

    if job[:reject_reason].nil?
      job[:reject_reason] = 'manualReject'
      job[:reject_description] = 'job was rejected manually'
      job.job_histories.create(event_type: 'manual_reject', active: true, reject_reason: 'manual_reject')
    else
      job[:reject_reason] = nil
      job[:reject_description] = nil
      job.job_histories.create(event_type: 'manual_unreject', active: true)
    end

    job.last_update_reject_time = Time.now
    if job.save
      job.remove_from_inf_queue
      if job[:active]
        JobRouter::Client.job_set_reject(job)
      else
        job[:active] = true
        job.save
        json = job.get_json
        last_non_estimated_job_history = job.job_histories.where(is_estimated: false).last
        json[:location] = {node: last_non_estimated_job_history[:location], connector: last_non_estimated_job_history[:connector]}
        json[:jobNumberAliases] = job[:job_numbers].nil? ? [] : JSON.parse(job[:job_numbers])
      end
      render json: job
    else
      render json: job.errors, status: :unprocessable_entity
    end
  end

  def revive
    @model = Job.find_by(id: params[:job_id])
    begin
      resp = RxModule::Communicator.call_module_method('rx_data', "/api/jobs/#{2}/revive", :patch)
      if resp.nil?
        Rails::logger.warn "Tried to revive a job that is either archived or doesnt exist"
        render json: {error: 'Tried to revive a job that is either archived or doesnt exist'}, status: :unprocessable_entity
        return
      end
      @rx_data_job_status = {}
      @rx_data_job_status[@model.number] = resp[:status]
    rescue => e
      render json: {status: 'Rx-data unreachable'}, status: :unprocessable_entity
      return
    end
    render :show
  end

  def update_required_processes
    processes_selected, capabilities_selected = required_processes_params

    required_processes_to_broadcast = []

    # Add to the history the processes the job has before the change
    @model.job_histories.create(event_type: 'manual_set_processes(before)', state: @model.process_snapshot)
    required_processes_to_broadcast += @model.set_processes_for_side(processes_selected[0], capabilities_selected[0], 'left')
    required_processes_to_broadcast += @model.set_processes_for_side(processes_selected[1], capabilities_selected[1], 'right')
    @model.job_histories.create(event_type: 'manual_set_processes(after)', state: @model.process_snapshot)
    @model.remove_from_inf_queue

    @model.un_reject if @model.is_rejected?
    JobRouter::Client.job_set_processes(@model)

    #TODO: not sure why we're manually broadcasting everything manually, something to investigate when we have more time
    ActionCable.server.broadcast "everything", {
      model: JSON.parse(@model.render_jbuilder_partial),
      class: @model.class.name,
      action: :update
    }

    required_processes_to_broadcast.each do |required_process_to_broadcast|
      ActionCable.server.broadcast "everything", {
        model: JSON.parse(required_process_to_broadcast.render_jbuilder_partial),
        class: required_process_to_broadcast.class.name,
        action: :update
      }
    end

    render :show, status: :ok
  end


  # TODO: no longer used now that rx-routing does a synchrounous call for the job id
  # Investigate to remove this as of 2022/11/9
  def job_create_rx_data_update
    params = rx_data_job_create_params
    job = Job.find_by(number: params[:number])
    unless job.nil?
      unless job.rx_data_job_id
        job.rx_data_job_id = params[:id]
        job.save
      end
    end
    send_push_response
  end

  def rx_data_update
    job_params, machine_params, job_state = job_state_params
    UpdateJobStateJob.perform_later(job_params, machine_params, job_state)
    send_push_response
  end

  def job_history
    filter_ids = job_history_params
    return render json: @model.job_histories.to_json if filter_ids.nil?
    render json: @model.job_histories.where.not(id: filter_ids).to_json
  end

  def to_json(*args)
    render_jbuilder_partial
  end

  private

  def create_params
    params.permit(:job_number)
  end

  def job_state_params
    job = params.require(:job).permit(:number)
    machine = params.require(:machine).permit(:sn, :name)
    job_state = params.permit( :process, :side, :state, :time, :comment, :scrap_reason, :substate)
    [job, machine, job_state]
  end

  def rx_data_job_create_params
    params.permit(:id, :number)
  end

  def required_processes_params
    job_processes_selected = params.fetch(:requiredProcessesSelected)
    capabilities_selected = params.fetch(:capabilitiesSelected)
    return job_processes_selected, capabilities_selected
  end

  def job_history_params
    filter_ids = params.fetch(:filter_ids)
    return filter_ids
  end

  def targets_external_id
    @models.pluck(:number, :rx_data_job_id).map do |job|
      {
        job_number: job[0],
        rx_data_job_id: job[1]
      }
    end
  end
end