class MachinesController < RxModule::ApplicationController
  include PushDataResponse
  include IsComponent
  model_data_store(Machine)

  @last_state_update_time = nil
  UPDATE_DATA_STATE_THRESHOLD = 60.seconds

  class << self
    def allowed_to_update_states_from_data?
      allowed = false
      allowed = true if !@last_state_update_time.nil? && (Time.zone.now - @last_state_update_time > UPDATE_DATA_STATE_THRESHOLD)
      allowed = true if @last_state_update_time.nil?
      @last_state_update_time = Time.zone.now if allowed
      allowed
    end
  end

  def before_set_targets
    # both left_outer_joins because we still want machines without job_process or capabilities
    # sub with device classname and the name of the component relationship
    Machine.joins(:component).left_outer_joins(:job_process).left_outer_joins(:capabilities).distinct
  end

  def before_filter(filter)
    if filter[:id] == 'name'
      filter[:id] = 'component.name'
    elsif filter[:id] == 'job_process'
      filter[:id] = 'job_process.name'
    elsif filter[:id] == 'capability_ids'
      filter[:id] = 'capabilities.rank'
    end
    filter
  end

  def before_sort(filter)
    if filter[:id] == 'name'
      filter[:id] == 'component.name'
    elsif filter[:id] == 'job_process'
      filter[:id] = 'job_process.name'
    elsif filter[:id] == 'capability_ids'
      filter[:id] = 'capabilities.name'
    end
    filter
  end

  def create
    params = model_params
    if Setting.use_rx_data_machines?
      machine_class = MesMachine
      component_class = MesComponent
      connector_class = MesConnector
    else
      machine_class = OpcuaMachine
      component_class = OpcuaComponent
      connector_class = OpcuaConnector
    end
    component = component_class.new(name: params[:name])
    machine = machine_class.new(ignore_state: params[:ignore_state], force_duration: params[:force_duration])
    component.machine = machine
    machine.job_process = JobProcess.find(params[:job_process_id])
    unless params[:capability_ids].blank?
      capabilities = Capability.where(id: params[:capability_ids])
      machine.capabilities << capabilities
    end
    machine_connectors = [
      {
          name: 'input',
          connector_type: 'input',
          state: 'online'
      },
      {
          name: 'output',
          connector_type: 'output',
          state: 'online'
      }
    ]
    machine_connectors.each do |machine_connector|
      connector = connector_class.new(machine_connector)
      machine.component.connectors << connector
    end

    if machine.save
      render json: machine
    else
      render json: machine.errors, status: :unprocessable_entity
    end
  end

  def index
    Machine.update_rx_data_machines if MachinesController.allowed_to_update_states_from_data?
  end

  def update
    params = model_params
    begin
      machines_to_update = params[:update_machine_group] ? @model.machines_in_same_group : [@model]
      machines_to_update.each do |machine|
        machine.update_from_params(params)
      end
    rescue
      return render json: {error: "Something went wrong while updating the machine group."}, status: :internal_server_error
    end

    # Setting that apply only to the current machine, even if update machine group is true
    if JobRouter::Client.is_offline?
      @model.component.name = params[:name] unless @model.is_mes_machine
    end

    render json: machines_to_update
  end

  def destroy
    if @model.destroy
      render json: @model
    else
      render json: @model.errors, status: :unprocessable_entity
    end
  end

  def reject_stations
    failed = false
    processed_job_process = JobProcess.find_by_name('processed')
    if processed_job_process.nil?
      render json: nil
      return
    end
    rejected_capability = Capability.find_by_name('rejected')
    if rejected_capability.nil?
      render json: nil
      return
    end
    process_id = processed_job_process[:id]
    capability_id = rejected_capability[:id]
    processed_machines = Machine.where(job_process_id: process_id)
    rejected_machines = []
    processed_machines.each do |machine|
      if machine.capability_ids.include? capability_id
        rejected_machines << {
            id: machine[:id],
            name: machine.component[:name]
        }
      end
    end
    render json: rejected_machines
  end

  # A machine was created/updated/deleted in rx-data
  def change_from_external
    params = rx_data_change_params
    #If changes is for a non conveyor machine in rx-data, ignore
    if params[:is_conveyor_machine]
      if params[:destroyed] == true
        machine = MesMachine.find_by_rx_data_id(params[:id])
        unless machine.nil?
          Rails.logger.warn "Machine #{machine.component[:name]} was deleted in rx-data."
          #TODO: this should probably destroy the machine here too?
        end
      else
        if MesMachine.exists?(sn: params[:sn])
          machine = MesMachine.find_by_sn(params[:sn])
          machine.component[:name] = params[:name]
          machine.save
        else
          Machine.create_from_rx_data(params)
        end
      end
    end
    send_push_response
  end

  def state_from_external
    params = rx_data_state_params
    # Since some machines have no SN, ensure an SN is specified
    unless params[:machine_sn].nil?
      machine = MesMachine.find_by_sn(params[:machine_sn])
      unless machine.nil?
        machine.last_reported_state = params[:state].upcase
        machine.save!
      end
    end
    send_push_response
  end

  def update_state
    params = model_params
    machine = Machine.find(params[:id])
    if machine[:sn].nil?
      machine.component.connectors.update_all(state: params[:state])
      if machine.save
        JobRouter::Client.machine_update(machine)
        json = machine.as_json
        json[:state] = machine.machine_state
        render json: json
      else
        render json: machine.errors, status: :unprocessable_entity
      end
    else
      machine[:forced_state] = params[:forced_state]
      if machine.save
        JobRouter::Client.machine_update(machine)
        render json: machine
      else
        render json: machine.error, status: :unprocessable_entity
      end
    end
  end

  # Legacy endpoint to support an S4P endpoint that allowed machines to toggle capabilities.
  # This is in use at the Pionneer Lab in Asia
  def capability_update
    resp = {status: 'updated'}

    if params[:machine_sn].nil? || params[:machine_sn].nil?
      Rails::logger.warn "Failed to update machine capabilities, missing params."
      return render json: resp, status: :ok
    end

    machine = MesMachine.find_by_sn(params[:machine_sn])
    if machine.nil?
      Rails::logger.warn "Failed to update machine capabilities for machine SN #{params[:machine_sn]}. No such machine."
      return render json: resp, status: :ok
    end

    enabled_capabilities = params[:capabilities].map {|c| c[:name]}

    enabled = []
    disabled = []
    machine.capabilities.each do |capability|
      if enabled_capabilities.include? capability.name
        enabled << capability.id
      else
        disabled << capability.id
      end
    end

    machine.capability_machines.where(capability_id: enabled).update_all(enabled: true) unless enabled.empty?
    machine.capability_machines.where(capability_id: disabled).update_all(enabled: false) unless disabled.empty?

    JobRouter::Client.machine_update(machine)

    render json: resp, status: :ok
  end

  def to_json(*args)
    render_jbuilder_partial
  end

  private

  def model_params
    params.fetch(:machine).permit( :id,
                                   :name,
                                   :force_duration,
                                   :job_process_id,
                                   :state,
                                   :update_machine_group,
                                   :ignore_state,
                                   :selected_reject,
                                   :forced_state,
                                   :apply_process_change_on,
                                   :allow_job_start,
                                   :restart_jobs,
                                   :lms_config_id,
                                   :capability_ids => [])
  end

  def rx_data_change_params
    if params[:destroyed] == true
      params.permit( :id, :destroyed)
    else
      params.permit( :id, :sn, :name, :description, :is_conveyor_machine, :action, :machine_group_ids => [])
    end
  end

  def rx_data_state_params
    params.permit( :machine_sn, :state)
  end
end
