class RouterServerController < ApplicationController
  include RouterResponse

  class SettingsNotConfigured < StandardError; end
  class RoutingGraphNotConfigured < StandardError; end

  ### GENERAL
  def status
    sub_systems = []

    sub_systems.push({name: 'Rx-Data', online: JobRouterControl.first.rx_data_online}) if Setting.use_rx_data?

    if Setting.lms_tag_source?
      LmsConfig.all_job_lms.each do |lms_config|
        sub_systems << {name: lms_config.name, online: lms_config.online}
      end
    end

    render json: {
      subSystems: sub_systems
    }, status: :ok
  end

  def state
    JobRouter::Client.set_router_state(state_params)
    send_router_response
  end

  def event
    #params = event_params
    #return Rails::logger.warn "Router Server gave warning: #{params[:message]}" if params[:level] == 'warning'
    #JobRouter::Client.client_error(params)
    send_router_response
  end

  ### STARTUP
  def base_settings
    do_startup_step(3, 'Initializing base settings', {
       options: {
           mode: ENV['RX_ROUTER_LEVEL'],
           plcController: ENV['PLC_CONTROL'],
           routingOptions: {
               markJobDeadAfterXVisit: settings.mark_job_dead_after_x_visit,
               loadBalancingMode: settings.load_balancing_mode,
               useRouteCache: settings.route_cache_enabled,
               useLoopClustering: settings.loop_clustering_enabled
           }
       }
   })
  end

  def plc_settings
    do_startup_step(4, 'Connecting to PLC controller', { options: plc_options })
  end

  def device_definitions
    do_startup_step(5, 'Preparing Device Types',{
        definitions: DeviceType.all.map do |device_type|
          device_type.get_json
        end
    })
  end

  def devices
    do_startup_step(6, 'Initializing Devices',{
        devices: Device.all.map do |device|
          device.get_json
        end
    })
  end

  def machines
    if connection_type_mes?
      MesMachine.update_machine_states
    end
    do_startup_step(7, 'Initializing Devices',{
        machines: routing_graph.routing_machines.map do |machine|
          if machine.job_process.nil?
            Rails.logger.warn "Machine #{machine.component.name} does not have a process"
          end
          machine.get_json
        end
    })
  end

  def processes
    do_startup_step(8, 'Initializing Devices',{
      # Do not include control processes
      processes: routing_graph.processes_offered.filter {|process| !process.is_control_process }.map do |process|
        process.get_json
      end
    })
  end

  def scanner_settings
    do_startup_step(9, 'Initializing Scanner Service', {
        config: settings.external_scanner_service_settings,
    })
  end

  def routes
    do_startup_step(10, 'Creating and Validating Routes',  {
        routes: routing_graph.get_json
    })
  end

  def default_job_processes
    do_startup_step(11, 'Generating Default Processes',  {
        processes: ProcessFetcher.default_processes_json
    })
  end

  private

  def do_startup_step(step, start_message, payload)
    JobRouter::Client.broadcast_startup_state(start_message, step)
    render json: payload, status: 200
  end

  def settings
    raise SettingsNotConfigured.new if Setting.first.nil?
    @settings ||= Setting.first
  end

  def routing_graph
    raise RoutingGraphNotConfigured.new if RoutingGraph.first.nil?
    @routing_graph ||= RoutingGraph.first
  end

  def connection_type_opcua?
    settings.connection_type == 'OPCUA'
  end

  def connection_type_mes?
    settings.connection_type == 'MES'
  end

  def plc_options
    return {
        opcuaServerURL: settings[:opcua_server_url],
        variant: settings[:variant],
        deviceRootPath: settings[:device_root_path],
    } if connection_type_opcua?

    {
        dataFormat: settings[:data_format]
    }
  end

  def broadcast_startup_state(message, step)
    max_step = 11.0
    progress = (step / max_step) * 100

    ActionCable.server.broadcast "router", {
        last_broadcast: "#{message}...",
        startup_progress: progress.floor
    }
  end

  def fetch_data
    params.fetch(:data)
  end

  def timestamp_param
    params.permit(:timestamp)
  end

  def state_params
    fetch_data.permit(:status, :message)
  end

  def event_params
    fetch_data.permit(:message, :level, :location => {})
  end
end
