import * as React from "react";
import {observer} from "mobx-react-lite";
import {TabPanel, TabPanelProps} from "../../generic/TabPanel";
import {ModelInput, ModelSelect, SelectField} from "@rx-suite/ui";
import {ConnectorModel} from "../../../stores/Device/ConnectorStore";
import {JobRouterControlStore} from "../../../stores/JobRouterControlStore";
import {ScannerModel, ScannerStore} from "../../../stores/ScannerStore";
import {t} from "@lingui/macro";
import {SettingStore} from "../../../stores/SettingStore";

type OpcuaConnectorRoutingLogicTabProps = {
    model: ConnectorModel
} & TabPanelProps

export const OpcuaConnectorScannerTab: React.FunctionComponent<OpcuaConnectorRoutingLogicTabProps> = observer((props) => {
    const {value, index, model} = props;
    const settings = SettingStore.selectedModel;

    const getScannerSource = (model: ConnectorModel) => {
        if(model.opcua_scanner_reference) { return 'opcua'}
        if(model.rx_scanner_reference) { return 'rx-scanners'}
        return 'none'
    }

    const scannerModels = settings?.use_rx_scanners ? ScannerStore.getAll() as ScannerModel[] : [];
    const [scannerSource, setScannerSource] = React.useState<string>(getScannerSource(model));
    const routerInfo = JobRouterControlStore.selectedModel;

    const defaultOpcuaVar = (): string => {
        let defaultName = 'SC';

        if(model.name == 'trunk-in'){
            defaultName += 'T'
        }else if(model.name == 'feed-in'){
            defaultName += 'F'
        }
        else{
            defaultName = '';
        }

        return defaultName;
    }

    const handleChangeScannerTarget = (name: string | undefined, value: string) => {
        if(value === 'opcua'){
            model.viewModel.opcua_scanner_reference = defaultOpcuaVar();
        }else{
            model.viewModel.opcua_scanner_reference = null;
        }

        if(value !== 'rx-scanners'){
            model.viewModel.rx_scanner_reference = null;
        }

        setScannerSource(value);
    }

    const scannerOptions = () => {
        let scanners = scannerModels.map((scanner) => {return {label: scanner.name, value: scanner.id}});
        scanners.push({label: 'none', value: null});
        return scanners;
    }

    const scannerTargets = () => {
        const choices =  [
            {label: 'None', value: 'none'},
            {label: 'OPCUA Node', value: 'opcua'}
        ]

        if(settings?.use_rx_scanners){
            choices.push({label: 'Rx-Scanners', value: 'rx-scanners'})
        }

        return choices;
    }

    if(model.viewModel.scanner_retry === null) {
        model.viewModel.scanner_retry = 3;
    }

    return (
        <TabPanel value={value} index={index}>
            <SelectField label={t`Scanner Target`} onChange={handleChangeScannerTarget} value={scannerSource} options={scannerTargets()}/>
            {scannerSource === 'rx-scanners' && <ModelSelect label={t`Scanner Name`} name={t`rx_scanner_reference`} options={scannerOptions()} disabled={!(routerInfo?.is_offline)}/>}
            {scannerSource === 'opcua' && <ModelInput label={t`OPCUA Variable`} name={'opcua_scanner_reference'} hint={{title: 'OPCUA Scanner Variable', text: 'This variable will be used to automatically calculate the id of the status and value fields. Ex: SCT will automatically generate SCTS (status field) and SCTV (value field).' }}/>}
            {scannerSource !== 'none' && <ModelInput name={'scanner_retry'}/>}
        </TabPanel>
    )
})