import React, {useEffect} from "react";
import {FormProps} from "@rx-suite/ui";
import {TagStore} from "../../stores/TagStore";
import {LinearProgress} from "@material-ui/core";
import {TagConditionStore} from "../../stores/TagConditionStore";
import {Builder, Query, Utils as QbUtils} from "react-awesome-query-builder";
import AntdConfig from 'react-awesome-query-builder/lib/config/antd';
import "antd/dist/antd.css"
import 'react-awesome-query-builder/css/styles.scss';
import {observer} from "mobx-react-lite";
import {SettingStore} from "../../stores/SettingStore";
import {JobRouterControlStore} from "../../stores/JobRouterControlStore";

interface TagQueryEditorProps extends FormProps {
    target: string,
    setStateCB: (tree: any, config: any) => void
}

const initialConfig = {
    ...AntdConfig,
    fields: {
    }
}

const renderBuilder = props => (
    <div className="query-builder-container" style={{padding: '10px'}}>
        <div className="query-builder qb-lite">
            <Builder {...props} />
        </div>
    </div>
);

export const TagQueryEditor: React.FunctionComponent<TagQueryEditorProps> = observer(({ model , target, setStateCB}) => {
    const [tree, setTree] = React.useState(null);
    const [config, setConfig] = React.useState(null);
    const [loaded, setLoaded] = React.useState(false);

    const settings = SettingStore.selectedModel
    const routerInfo = JobRouterControlStore.selectedModel
    const allTagConditions = TagConditionStore.getAll();
    let tags = TagStore.getAll();

    useEffect(() => {
        initialConfig.operators['contains'] = {
            label: 'contains',
            reversedOp: 'does not contain',
            cardinality: 1,
            jsonLogic: (field, op, value, opDef, operatorOptions) => {
                return {
                    'in' : [
                        value, field
                    ]
                }
            },
            //This is a hidden field that tells jsonlogic the operator cuz aparently it cant tell from the object above that the op is in
            //# is to tell is that its reversed, because we want value then field, since thats how in works
            jsonLogic2: '#in'
        }

        initialConfig.types['text']['widgets']['text']['operators'].push('contains')
    },[])

    useEffect(() => {
        if(!tags || !allTagConditions) {
            return
        }
        for (const tag of tags) {
            const tagName:string = tag.tag;
            const valueType = tag.value_type;
            if(valueType === 'number') {
                initialConfig.fields[tagName] = {
                    type: 'number',
                    valueSources: ['value'],
                    preferWidgets: ['number']
                }
            } else {
                initialConfig.fields[tagName] = {
                    type: 'text',
                    valueSources: ['value'],
                    preferWidgets: ['text']
                }
            }
        }

        /* Override default system tag options */
        initialConfig.fields['processes'] = {
            type: 'text',
            valueSources: ['value'],
            preferWidgets: ['text'],
            operators: ['contains']
        }

        initialConfig.fields['lens'] = {
            type: 'select',
            valueSources: ['value'],
            fieldSettings: {
                listValues: [
                    { value: 'left', title: 'left' },
                    { value: 'right', title: 'right' },
                ],
            }
        }

        const filteredStateConditions = allTagConditions.find((condition) => target === 'capability' ? condition.capability_id === model.id : condition.job_process_id === model.id);
        let logic = {and: []}
        if (filteredStateConditions && filteredStateConditions.condition && filteredStateConditions.condition.logic) {
            logic = filteredStateConditions.condition.logic
        }

        setConfig(initialConfig);
        const tree = QbUtils.checkTree(QbUtils.loadFromJsonLogic(logic, initialConfig), initialConfig)
        setTree(tree);
        setStateCB(tree, initialConfig);
        setLoaded(true);

    }, [tags, settings, allTagConditions]);

    const handleOnChange = React.useCallback((immutableTree, newConfig) => {
        setTree(immutableTree);
        setConfig(newConfig);
        setStateCB(immutableTree, newConfig)
    }, []);

    useEffect(() => {
        if(!settings) {
            return
        }
        setConfig((prev) => {
            if(prev) {
                prev.settings.immutableFieldsMode = !routerInfo?.is_offline;
                prev.settings.immutableGroupsMode = !routerInfo?.is_offline;
                prev.settings.immutableOpsMode = !routerInfo?.is_offline;
                prev.settings.immutableValuesMode = !routerInfo?.is_offline;
            }
            return prev
        })
    }, [settings])

    if (!loaded) {
        return <LinearProgress/>
    }

    return(
        <React.Fragment>
            <Query
                {...config}
                value={tree}
                onChange={handleOnChange}
                renderBuilder={renderBuilder}
            />
        </React.Fragment>)
});