import React, {useEffect} from "react";
import {FormProps, PaddedPaper} from "@rx-suite/ui";
import {observer} from "mobx-react-lite";
import {Box, Button, CircularProgress, Grid, makeStyles, Typography} from "@material-ui/core";
import {AddCircle} from "@material-ui/icons";
import {t, Trans} from "@lingui/macro";
import {JobFinderScannerField} from "./jobFinderScannerField";
import {Alert} from "@material-ui/lab";
import {grey} from "@material-ui/core/colors";
import {useSnackbar} from "notistack";
import {RxAppModel, RxAppStore} from "@rx-suite/stores";
import {routerStore} from "@rx-suite/data";

const useStyles = makeStyles((theme) => ({
    searchContainer: {
        marginBottom: theme.spacing(2),
        backgroundColor: 'rgb(250,218,105)'
    },
    searchBox: {
        width: '90%',
        display: 'inline'
    },
    text: {
        color: grey[700]
    },
    icon: {
        color: grey[700],
        marginRight: theme.spacing(1)
    }
}));

export const JobCreateInput: React.FunctionComponent<FormProps> = observer(({model}) => {

    const {enqueueSnackbar} = useSnackbar();
    const classes = useStyles();
    const appModel = RxAppStore.selectedModel as RxAppModel;
    const [key, setKey] = React.useState(Math.random());
    const [jobNumber, setJobNumber] = React.useState<string>('');
    const [scannedJobNumber, setScannedJobNumber] = React.useState<string>('');
    const [isFetching, setIsFetching] = React.useState(false);

    useEffect(() => {
        window.addEventListener("keypress", (event) => {
            setKey(Math.random());
        })
    }, []);

    useEffect(() => {
        setKey(Math.random());
    }, [appModel.drawerOpen]);

    const handleJobFinder = async () => {
        let error ="";
        setIsFetching(true);
        try {
            const number = scannedJobNumber !== '' ? scannedJobNumber : jobNumber
            if(number === '')  {
                enqueueSnackbar(`${t`Cant search for a job without a number`}`, {variant: "error"});
                setIsFetching(false);
                return
            }
            const resp = await fetch(`/api/jobs`, {
                method: "POST",
                headers: {
                    "Content-Type": "application/json"
                },
                body: JSON.stringify({job_number: number})
            })
                .then(resp => resp.json())
                .then(data => {
                    console.log(data);
                    if(data.id) {
                        enqueueSnackbar(`${t`Job Created`}`, {variant: "success"});
                        routerStore.history.push(`/job_workstation/jobs/${data['id']}/edit?jobCreate=true`);
                    } else {
                        enqueueSnackbar(`${t`Job did not exist in the data source system. Job created anyways`}`, {variant: "error"});
                    }
                    setScannedJobNumber('');
                    setJobNumber('');
                });
        } catch (err) {
            error = err;
            console.error(error);
            enqueueSnackbar({error}, {variant: "error"});
        }
        setIsFetching(false);
    }

    const onChange = (e: any) => {
        setJobNumber(e.target.value);
    }

    const onEnterKey = async () => {
        if(isFetching){
            return;
        }
        await handleJobFinder();
    }

    const onComplete = async (code: string) => {
        setScannedJobNumber(code);
    }

    return (
        <div>
            <PaddedPaper className={classes.searchContainer}>
                <Grid container alignItems={'center'}>
                    <AddCircle  className={classes.icon}/>
                    <Typography variant={'h5'} className={classes.text}><Trans>Create Job For Routing</Trans></Typography>
                </Grid>
                <Grid container justifyContent="space-evenly">
                    <Grid className={classes.searchBox} item>
                        <JobFinderScannerField  jobNumber={jobNumber} onChange={onChange} onEnterKey={onEnterKey} onComplete={onComplete} key={key}/>
                    </Grid>
                    <Button type='submit' color='secondary' onClick={handleJobFinder} variant='contained' disabled={isFetching}>
                        {isFetching ? <CircularProgress /> : <Trans>Create</Trans>}
                    </Button>
                </Grid>
                <Box mt={1}>
                    <Alert severity='warning'>
                        <Trans>Caution: Creating a job will overwrite any existing job with the same number.</Trans>
                    </Alert>
                </Box>
            </PaddedPaper>
            {model?.reject_reason &&
                <Alert severity='error'>
                    <Trans>Error: Could not find data for this job. Routing to reject.</Trans>
                </Alert>
            }
        </div>
    )
})