import React, {useEffect} from "react";
import {FormProps, ModelProvider} from "@rx-suite/ui";
import {observer} from "mobx-react-lite";
import {Alert} from "@material-ui/lab";
import {t, Trans} from "@lingui/macro";
import {Button, Grid, Tab, Tabs, Typography} from "@material-ui/core";
import {ProcessInfoPanel} from "./panels/ProcessInfoPanel";
import {JobModel} from "../../stores/JobStore";
import {JobHistoryPanel} from "./panels/JobHistoryPanel";
import {SettingStore} from "../../stores/SettingStore";
import {useSnackbar} from "notistack";

export const JobInfo: React.FunctionComponent<FormProps> = observer(({model}) => {

    const {enqueueSnackbar} = useSnackbar();

    const settings = SettingStore.selectedModel

    const [jobFinderKey, setJobFinderKey] = React.useState<number>(Math.random());
    const [tabValue, setTabValue] = React.useState(0);
    const [jobStatus, setJobStatus] = React.useState<string | null>(null);

    const fetch_job_state = async () => {
        try {
            const resp = await fetch(`/api/jobs/${model.id}/fetch_job_status`, {
                method: "GET",
                headers: {'Content-Type': 'application/json'}
            }).then(resp => resp.json())
                .then(data => {
                    setJobStatus(data['status']);
                })
        } catch (err) {
            console.error(err);
            enqueueSnackbar({err}, {variant: "error"});
        }
    }

    useEffect(() => {
        if(settings) {
            if(settings?.connection_type === "MES") {
                fetch_job_state()
            }
        }
    }, [settings])

    const handleReviveOnClick = React.useCallback(async () => {
        try {
            console.log(`revive job id: ${model.id}`);
            const resp = await fetch(`/api/jobs/${model.id}/revive`, {
                method: "PATCH",
                headers: {'Content-Type': 'application/json'}
            })
            const data = await resp.json();
            if(!resp.ok) {
                enqueueSnackbar(`${data.error}`, {variant: "error"})
            } else {
                enqueueSnackbar(`${t`Job Revived`}`, {variant: "success"})
                fetch_job_state();
            }
        } catch (err){
            console.error(err);
            enqueueSnackbar({err}, {variant: "error"});
        }
    }, [model.id]);

    const handleRejectOnClick = React.useCallback(async () => {
        try {
            console.log(`set reject job id: ${model.id}`);
            const resp = await fetch(`/api/jobs/${model.id}/reject`, {
                method: "POST",
                headers: {'Content-Type': 'application/json'}
            }).then(resp => resp.json())
            .then(data => {
                enqueueSnackbar(`${t`Job state updated`}`, {variant: "success"})
            })
        } catch (err){
            console.error(err);
            enqueueSnackbar({err}, {variant: "error"});
        }
    }, [model.id]);

    const rejectButtonText = () => {
        if(model.reject_reason) {
            return t`Unreject job`;
        } else {
            return t`Reject job`;
        }
    }

    const handleTabChange = (event: any, newValue: React.SetStateAction<number>) => {
        setTabValue(newValue);
    };

    return (
        <ModelProvider model={model}>
            {model.inf_target_machine_id && <Alert severity="warning"><Trans>This job is waiting to receive INF data from an LMS.</Trans></Alert>}
            <Grid container >
                <Grid item xs={12}>
                    <Typography variant={'h4'} color={'primary'}>
                        <Trans>
                            Job Number: {model.number}
                       </Trans>
                    </Typography>
                    <br/>
                    {model.reject_reason &&
                    <Typography variant={'h4'} color={'error'}>
                        <Trans>
                            Reject reason: {model.reject_reason}
                        </Trans>
                    </Typography>}
                    {model.reject_description &&
                    <Typography variant={'h6'} color={'error'}>{model.reject_description}</Typography>}
                    <div>
                        {jobStatus &&
                            <Typography variant={'h4'} color={'primary'} style={{display: 'inline-block'}}>
                                <Trans>
                                    Status: {jobStatus}
                                </Trans>
                            </Typography>
                        }
                        {jobStatus === 'expired' &&
                        <Button variant={'contained'} onClick={handleReviveOnClick} color={"secondary"} style={{margin: 10}}>
                            <Trans>
                                Reactivate
                            </Trans>
                        </Button>
                        }
                    </div>
                    <Button variant="outlined" color="secondary" onClick={handleRejectOnClick}>
                        {rejectButtonText()}</Button>
                </Grid>
            </Grid>
            <Tabs
                value={tabValue}
                onChange={handleTabChange}
                indicatorColor="primary"
                textColor="primary"
            >
                <Tab label={t`Process Info`}/>
                <Tab label={t`Job History`}/>
            </Tabs>
            <ProcessInfoPanel job={model as JobModel} index={0} value={tabValue} setJobFinderKey={setJobFinderKey}/>
            <JobHistoryPanel job={model as JobModel} index={1} value={tabValue}/>
        </ModelProvider>
    )

})
