import {TabPanel, TabPanelProps} from "../tabPanel";
import {observer} from "mobx-react-lite";
import ReactTable from "react-table";
import React, {useEffect} from "react";
import {makeColumn, makeFormattedDateTimeColumn} from "@rx-suite/ui";
import { t } from "@lingui/macro";
import {JobHistoryModel, JobHistoryStore} from "../../../stores/JobHistoryStore";
import {JobModel} from "../../../stores/JobStore";
import {Refresh} from "@material-ui/icons";
import {Button, createStyles, FormControlLabel, Grid, Switch, Theme, Typography} from "@material-ui/core";
import moment from "moment";
import {makeStyles} from "@material-ui/core/styles";

type PanelProps = {
    job: JobModel;
} & TabPanelProps

const useStyles = makeStyles((theme: Theme) =>
    createStyles({
        switch: {
            "& .MuiIconButton-root" : {
                padding: '9px',
            }
        }
    })
);

export const JobHistoryPanel: React.FunctionComponent<PanelProps> = observer((props) => {
    //const [sorts, setSorts] = React.useState([{ id: 'created_at', desc: false}])

    const classes = useStyles();

    const [jobHistoryModels, setJobHistoryModels] = React.useState<ReadonlyArray<JobHistoryModel>>([])

    const [jobHistoryIds, setJobHistoryIds] = React.useState<ReadonlyArray<number>>([]);

    const [isAutoRefresh, setIsAutoRefresh] = React.useState<boolean>(false);

    const [refreshing, setRefreshing] = React.useState<boolean>(false);

    const [lastRefresh, setLastRefresh] = React.useState<string>('');

    // hack to get a promise inside of useEffect (leverage react state)
    const fetchJobHistoriesForJob = async () => {

        setRefreshing(true);

        const job_histories = await props.job.refreshHistory(jobHistoryIds);

        if (!job_histories || job_histories.error) {
            setRefreshing(false);
            setLastRefresh(moment().format('DD/MM/YYYY - HH:mm:ss'));
            return;
        }

        const models = job_histories.map((job_history: JobHistoryModel) => {
            const jobHistoryModel = JobHistoryStore.createModel(job_history);
            jobHistoryModel.processes = JSON.parse(job_history.processes);
            return jobHistoryModel;
        });

        setJobHistoryModels(jobHistoryModels.concat(models));
        setJobHistoryIds(jobHistoryIds.concat(job_histories.map((job_history: JobHistoryModel) => job_history.id)));
        setRefreshing(false);
        setLastRefresh(moment().format('DD/MM/YYYY - HH:mm:ss'));
    };

    // Run once on mount
    useEffect(() => { fetchJobHistoriesForJob(); }, []);

    // isAutoRefresh is false on mount, so no risk of running the endpoint twice on mount
    useEffect(() => {

        // Don't auto refresh unless explicitly set
        if (isAutoRefresh) {
            fetchJobHistoriesForJob();
        }

    }, [props.job]);

    const handleAutoRefresh = (e) => {
        setIsAutoRefresh(e.target.checked);
    };

    //Since customSort is called before onSortedChange, I'm only allowing sorts on the created_at column.
    const customSort = (a:JobHistoryModel, b:JobHistoryModel) => {
        //for(const sort of sorts) {
            const timeA = a.time;
            const timeB = b.time;
            return new Date(timeA) - new Date(timeB);
        //}
    }

    /*const onSortedChange = (newSorts:{id:string, desc:boolean}[]) => {
        setSorts(newSorts);
    }*/
    const JobHistoryColumns = React.useMemo(() => {
        return [
            {
                Header: t`Job History`, columns:
                    [
                        makeColumn('event_type', t`Event type`, "string"),
                        makeColumn('location', t`Event location`, "string"),
                        makeColumn('connector', t`Event connector`, "string"),
                        makeColumn('state', t`State`, "string"),
                        makeColumn('reject_reason', t`Reject Reason`, "string"),
                        makeFormattedDateTimeColumn('time', t`Update time`, "dateTime"),
                    ]
                }
            ]
    }, [])

    return (
        <TabPanel value={props.value} index={props.index}>
            <Grid container alignItems="flex-end" justifyContent="space-between">
                <Grid item>
                    <Typography variant={'subtitle2'}>
                        {t`Last Successful Refresh:` + ' ' + lastRefresh}
                    </Typography>
                    <FormControlLabel
                        control={<Switch className={classes.switch} checked={isAutoRefresh} color={'secondary'} onChange={handleAutoRefresh}/>}
                        label="Auto-Refresh"
                    />
                </Grid>
                <Grid item>
                    <Button
                        variant="contained"
                        color="default"
                        size={"small"}
                        style={{
                            paddingTop: '8px',
                            paddingBottom: '8px',
                            marginBottom: '1rem'
                        }}
                        onClick={fetchJobHistoriesForJob}
                    >
                        <Refresh fontSize={"small"}/>
                    </Button>
                </Grid>
            </Grid>
            <ReactTable
                sortable={false}
                data={jobHistoryModels}
                columns={JobHistoryColumns}
                minRows={0}
                defaultPageSize={100}
                loading={refreshing}
                //sorted={sorts}
                //onSortedChange={onSortedChange}
                defaultSorted={[{ id: 'time', desc: true}]}
                defaultSortMethod={customSort}
                getTrProps={(state, rowInfo, instance) => {
                    if (rowInfo) {
                        return {
                            style: {
                                background: rowInfo.row.state.is_estimated ? 'rgb(255,255,255)' : 'rgb(192, 192, 192)'
                            }
                        }
                    }
                    return {}
                }}
            />
        </TabPanel>
    );
});