import React, {useEffect} from "react";
import {observer} from "mobx-react-lite";
import {JobModel} from "../../../stores/JobStore";
import {TabPanel, TabPanelProps} from "../tabPanel";
import {Model} from "@rx-suite/data";
import {
    Button,
    createStyles,
    Grid,
    IconButton,
    LinearProgress,
    Theme,
    Typography,
    useMediaQuery
} from "@material-ui/core";
import {Conditional, IfTrue, MaterialFAIcon, ModelProvider, ModelTableDefaultActions} from "@rx-suite/ui";
import {faSquareFull, faThumbtack} from "@rx-suite/pro-solid-svg-icons";
import classNames from "classnames";
import {RequiredProcessButtons} from "../../required_processes/RequiredProcessButtons";
import {RequiredProcessStore} from "../../../stores/RequiredProcessStore";
import { t, Trans } from "@lingui/macro";
import {makeStyles} from "@material-ui/core/styles";
import {RequiredProcessDialogComponent} from "../../required_processes/RequiredProcessDialogComponent";
import {useSnackbar} from "notistack";
import ReactTable from "react-table";
import {RequiredProcessStateHistoryTable} from "../../RequiredProcessStateHistoryTable";
import {FontAwesomeIcon} from "@fortawesome/react-fontawesome";
import {SettingModel, SettingStore} from "../../../stores/SettingStore";
import {SystemSettingModel, SystemSettingStore} from "@rx-suite/stores";

type PanelProps = {
    job: JobModel;
    setJobFinderKey: (a:number) => void;
} & TabPanelProps

const useStyles = makeStyles((theme: Theme) =>
  createStyles({
    FixedHeightRow: {
        height: '4em'
    },
    actions: {
        display: "flex"
    },
    pinnedActions: {
        borderLeftWidth: '2px',
        borderLeftColor: 'rgb(0,0,0)'
    }
  }),
);

const LegendIcon = ({label, color}) => {
    return (
        <Grid item style={{marginRight: 10}}>
            <Typography display={'inline'}>
                <Trans>
                    {label}
                </Trans>
            </Typography>
            <IconButton disabled={true} size={"small"}>
                <FontAwesomeIcon icon={faSquareFull} color={color} style={{border: '1px solid black'}}/>
            </IconButton>
        </Grid>
    )
}

export const ProcessInfoPanel: React.FunctionComponent<PanelProps> = observer((props) => {
    const {job, value, index, setJobFinderKey} = props;

    const {enqueueSnackbar} = useSnackbar();
    const classes = useStyles();

    const bigScreen = useMediaQuery(theme => theme.breakpoints.up('sm'));
    const [actionsPinned, setActionsPinned] = React.useState(true);
    const [rightExpanded, setRightExpanded] = React.useState<{number: boolean} | {}>({});
    const [leftExpanded, setLeftExpanded] = React.useState<{number: boolean} | {}>({});
    const [open, setOpen] = React.useState(false);
    const [loadingFlag, setLoadingFlag] = React.useState<boolean>(false);

    const systemSettingModel = SystemSettingStore.selectedModel as SystemSettingModel;
    const settings = SettingStore.selectedModel as SettingModel;

    const displayRightLensFirst = systemSettingModel?.displayRightLensFirst();

    let leftJobNumber = '';
    if(job.job_numbers && job.job_numbers !== '' && job.job_numbers !== '[]') {
        const json = JSON.parse(job.job_numbers)
        if(json.length == 2) {
            leftJobNumber = json[0]['jobNumber']
        } else if(json[0]['lens'] === 'left') {
            leftJobNumber = json[0]['jobNumber']
        }
    }

    let rightJobNumber = '';
    if(job.job_numbers && job.job_numbers !== '' && job.job_numbers !== '[]') {
        const json = JSON.parse(job.job_numbers)
        const rightObj = json.find((obj: {jobNumber: number, lens: 'left'|'right'}) => obj.lens === 'right');
        const leftObj = json.find((obj: {jobNumber: number, lens: 'left'|'right'}) => obj.lens === 'left');

        if(rightObj) {
            rightJobNumber = rightObj['jobNumber'];
        }
        if(leftObj) {
            leftJobNumber = leftObj['jobNumber'];
        }
    }

    const leftRequiredProcess = RequiredProcessStore.models.filter((process) => job.required_process_ids.includes(process.getId()) && process.side === 'left').sort((processA, processB) => processA.job_process.rank - processB.job_process.rank)
    const rightRequiredProcess = RequiredProcessStore.models.filter((process) => job.required_process_ids.includes(process.getId()) && process.side === 'right').sort((processA, processB) => processA.job_process.rank - processB.job_process.rank)

    useEffect(() => {
        const resp = RequiredProcessStore.getByIds(job.required_process_ids)
        resp.then(() => {
            setLoadingFlag(true);
        });
    }, [job.required_process_ids])

    useEffect(() => {
        if(rightRequiredProcess) {
            setRightExpanded(rightRequiredProcess.map((value, index) => {return false}));
        }
        if(leftRequiredProcess) {
            setLeftExpanded(leftRequiredProcess.map((value, index) => {return false}))
        }
    }, [])

    const colors = {
        notStarted: 'rgb(240,240,240, .4)',
        started: 'rgb(255,225,35, .4)',
        finished: 'rgb(60,183,29, .4)',
        aborted: 'rgb(239,55,19, .4)',
        rework: 'rgba(100,0,255,0.2)'
    }

    const getDisplayName = (val): string => {
        return !val.display_name ? val.name : val.display_name;
    };

    const LeftRequiredProcessColumns = React.useMemo(() => {return [{Header: t`Required Processes - Left -` + ' ' + t`Job number: ` + ' ' + leftJobNumber, columns: [
        {Header: t`Name`, id: 'processName', accessor: row => getDisplayName(row.job_process), sortable: false, filterable: false, minWidth: 75},
        {Header: t`Capabilities`, id: 'capabilities', accessor: 'capabilities', sortable: false, filterable: false, minWidth: 75, show: bigScreen,
            Cell: (cellInfo: { value: Model }) =>
            {return cellInfo.value ? cellInfo.value.map((value) => getDisplayName(value)).join(', ') : ""
        }},
        {Header: t`State`, id: 'state', accessor: 'state', minWidth: 85, sortable: false, filterable: false, show: false},
        {Header: t`Is Rework?`, id: 'is_rework', accessor: 'is_rework', sortable: false, filterable: false, minWidth: 85, show: false}
        ]},
        {
            fixed: 'right',
            columns: [
                {Header: () =>
                    (<IconButton onClick={() => setActionsPinned(!actionsPinned)}>
                        <MaterialFAIcon icon={faThumbtack} color={actionsPinned ? 'secondary' : undefined}/>
                    </IconButton>),
                accessor: d => d,
                id: '_actions',
                style: {textAlign: "right"},
                width: 200,
                sortable: false,
                filterable: false,
                resizable: false,
                Cell: (cellInfo: { value: Model }) => {
                    if (!cellInfo.value || !cellInfo.value.getId) {
                        return null;
                    }
                    return <div className={classNames(classes.actions, actionsPinned && classes.pinnedActions)}>
                        <RequiredProcessButtons model={cellInfo.value} setJobFinderKey={setJobFinderKey}/>
                        {bigScreen &&
                            <ModelTableDefaultActions store={RequiredProcessStore} model={cellInfo.value} replacements={{Delete: () => null}}/>
                        }
                    </div>}}
                ]}
    ]}, [bigScreen])

    const RightRequiredProcessColumns = React.useMemo(() => {return [{Header: t`Required Processes - Right -` + ' ' + t`Job number: ` + ' ' + rightJobNumber, columns: [
        {Header: t`Name`, id: 'processName', accessor: row => getDisplayName(row.job_process), sortable: false, filterable: false, minWidth: 75},
        {Header: t`Capabilities`, id: 'capabilities', accessor: 'capabilities', sortable: false, filterable: false, minWidth: 75, show: bigScreen,
            Cell: (cellInfo: { value: Model }) =>
            {return cellInfo.value ? cellInfo.value.map((value) => getDisplayName(value)).join(', ') : ""
        }},
        {Header: t`State`, id: 'state', accessor: 'state', sortable: false, filterable: false, minWidth: 85, show: false},
        {Header: t`Is Rework?`, id: 'is_rework', accessor: 'is_rework', sortable: false, filterable: false, minWidth: 85, show: false}
        ]},
        {
            fixed: 'right',
            columns: [
                {Header: () =>
                    (<IconButton onClick={() => setActionsPinned(!actionsPinned)}>
                        <MaterialFAIcon icon={faThumbtack} color={actionsPinned ? 'secondary' : undefined}/>
                    </IconButton>),
                accessor: d => d,
                id: '_actions',
                style: {textAlign: "right", textOverflow: "unset"},
                width: 200,
                sortable: false,
                filterable: false,
                resizable: false,
                Cell: (cellInfo: { value: Model }) => {
                    if (!cellInfo.value || !cellInfo.value.getId) {
                        return null;
                    }
                    return <div className={classNames(classes.actions, actionsPinned && classes.pinnedActions)}>
                        <RequiredProcessButtons model={cellInfo.value} setJobFinderKey={setJobFinderKey}/>
                        {bigScreen &&
                            <ModelTableDefaultActions store={RequiredProcessStore} model={cellInfo.value} replacements={{Delete: () => null}}/>
                        }
                    </div>
                }}
            ]
        }]}, [bigScreen])

    const requiredProcessTableTrProps = (state, rowInfo, instance) => {
        const hash = {
            className: classes.FixedHeightRow
        }
        if(rowInfo) {
            let color = 'rgb(255,255,255)';
            if(rowInfo.row.is_rework) {
                color = colors.rework;
            } else {
                switch(rowInfo.row.state) {
                    case 'NOTSTARTED':
                        color = colors.notStarted;
                        break;
                    case 'STARTED':
                        color = colors.started;
                        break;
                    case 'FINISHED':
                        color = colors.finished;
                        break;
                    case 'ABORTED':
                        color = colors.aborted;
                        break;
                }
            }
            hash['style'] = {
                background: color
            }
        }
        return hash
    }

    const onRightExpandedChange = (e, value) => {
        const tmp = {...rightExpanded};
        tmp[value[0]] = !tmp[value[0]];
        setRightExpanded(tmp);
    }

    const onLeftExpandedChange = (e, value) => {
        const tmp = {...leftExpanded};
        tmp[value[0]] = !tmp[value[0]];
        setLeftExpanded(tmp);
    }

    const minRows = () => {
        if(leftRequiredProcess.length > rightRequiredProcess.length) {
            return leftRequiredProcess.length;
        } else {
            return rightRequiredProcess.length;
        }
    }

    const handleClickOpen = () => {
        setOpen(true);
    }

    const handleClose = () => {
        setOpen(false);
    }

    const handleSubmit = React.useCallback(async (checked: {string: boolean}[], capabilitiesSelected: {string: string[]}[]) => {
        setOpen(false);
        try {
            const resp = await fetch(`/api/jobs/${job.id}/update_required_processes`, {
                method: "POST",
                headers: {'Content-Type': 'application/json'},
                body: JSON.stringify({requiredProcessesSelected: checked, capabilitiesSelected: capabilitiesSelected})
            }).then(resp => resp.json())
            .then(data => {
                enqueueSnackbar(`${t`Required processes successfully updated`}`, {variant: "success"})
            })
        } catch (err) {
            console.error(err);
            enqueueSnackbar({err}, {variant: "error"});
        }
    }, [job.id]);

    if(!loadingFlag) {
        return <LinearProgress color={'secondary'}/>
    }

    const table = (side: string) => {
        let data, columns, expanded, onExpandedChange = null;
        if(side == 'left') {
            data = leftRequiredProcess;
            columns = LeftRequiredProcessColumns;
            expanded = leftExpanded;
            onExpandedChange = onLeftExpandedChange;
        } else {
            data = rightRequiredProcess;
            columns = RightRequiredProcessColumns;
            expanded = rightExpanded;
            onExpandedChange = onRightExpandedChange;
        }
        return (
        <ReactTable
            data={data}
            columns={columns}
            minRows={minRows()}
            showPagination={false}
            sortable={false}
            filterable={false}
            defaultSorted={[{id: 'processRank', desc: false}]}
            getTrProps={(state, rowinfo, instance) => requiredProcessTableTrProps(state, rowinfo, instance)}
            getTdProps={(state, rowinfo, instance) => {
                return {style: {background: 'rgb(0,0,0,0)'}}
            }}
            SubComponent={bigScreen ? (row) => {
                return (
                    <RequiredProcessStateHistoryTable model={row.original}/>
                )
            } : undefined}
            expanded={expanded}
            onExpandedChange={onExpandedChange}
        />
        )
    }

    return (
        <TabPanel value={value} index={index}>
            <Button variant="outlined" color="secondary" onClick={handleClickOpen}>
                {t`Change Processes`}
            </Button>
            <RequiredProcessDialogComponent jobModel={job} open={open} handleSubmit={handleSubmit} handleClose={handleClose}/>
            <Grid container>
                <Grid item xs={6}>
                    {displayRightLensFirst ?
                        table('right') :
                        table('left')
                    }
                </Grid>
                <Grid item xs={6}>
                    {displayRightLensFirst ?
                        table('left') :
                        table('right')
                    }
                </Grid>
            </Grid>
            <Grid container alignItems={'flex-end'}>
                <LegendIcon label='NOTSTARTED' color={colors.notStarted}/>
                <LegendIcon label='STARTED' color={colors.started}/>
                <LegendIcon label='FINISHED' color={colors.finished}/>
                <Conditional condition={settings.isMESLevelRouting}>
                    <IfTrue><LegendIcon label='ABORTED' color={colors.aborted}/></IfTrue>
                </Conditional>
                <Conditional condition={settings.job_rework_enabled}>
                    <IfTrue><LegendIcon label='REWORK' color={colors.rework}/></IfTrue>
                </Conditional>
            </Grid>
        </TabPanel>
    );
});