import React, {useEffect} from "react";
import {observer} from "mobx-react-lite";
import {FormProps} from "@rx-suite/ui";
import {
    Button,
    createStyles,
    Dialog,
    DialogActions,
    DialogContent,
    DialogTitle,
    Grid,
    Theme,
    Typography
} from "@material-ui/core";
import { t, Trans } from "@lingui/macro";
import {RequiredProcessStore} from "../../stores/RequiredProcessStore";
import {JobProcessStore} from "../../stores/machine/JobProcessStore";
import {JobModel} from "../../stores/JobStore";
import {RequiredProcessCheckboxFactory} from "../RequiredProcessCheckboxFactory";
import {makeStyles} from "@material-ui/core/styles";
import {green, red} from "@material-ui/core/colors";
import {SystemSettingModel, SystemSettingStore} from "@rx-suite/stores";

interface RequiredProcessDialogProps extends FormProps{
    jobModel: JobModel,
    open: boolean,
    handleSubmit: (checked: {string: boolean}[], capabilitiesSelected: {string: string[]}[]) => void,
    handleClose: () => void
}

const useStyles = makeStyles((theme: Theme) =>
  createStyles({
        requiredProcessSide: {
            textAlign: 'center',
            fontWeight: 'bold'
        },
        addAll: {
            backgroundColor: green[300]
        },
        removeAll: {
            backgroundColor: red[300]
        }
  }),
);

export const RequiredProcessDialogComponent: React.FunctionComponent<RequiredProcessDialogProps> = observer(({jobModel, open, handleSubmit, handleClose}) => {

    const classes = useStyles();

    const [checked, setChecked] = React.useState<{string: boolean}[] | {}[] | any>([{}, {}]);
    const [capabilitiesSelected, setCapabilitiesSelected] = React.useState<{string: string[]}[] | {}[] | any>([{}, {}]);
    const [fixedJobProcesses, setFixedJobProcesses] = React.useState<[]>([]);

    const jobProcesses = JobProcessStore.getAll();
    const systemSettingModel = SystemSettingStore.selectedModel as SystemSettingModel;
    const displayRightLensFirst = systemSettingModel?.displayRightLensFirst();

    useEffect(() => {
        const resp = RequiredProcessStore.getByIds(jobModel.required_process_ids)
        resp.then((requiredProcessModels) => {
            let tmp = jobProcesses.filter((jobProcess) => !jobProcess.is_control_process);
            tmp = tmp.sort((jobProcessA, jobProcessB) => jobProcessA.rank - jobProcessB.rank)
            setFixedJobProcesses(tmp);
            tmp.forEach((jobProcess) => {
                for(let i = 0; i < 2; i++) {
                    const side = i === 0 ? 'left' : 'right';
                    if(jobModel.job_numbers !== "" && jobModel.job_numbers !== null) {
                        const job_numbers = JSON.parse(jobModel.job_numbers);
                        if(job_numbers.length < 2) {
                            if(job_numbers[0]['lens'] !== side) {
                                continue
                            }
                        }
                    }
                    const requiredProcess = requiredProcessModels.find((requiredProcessModel) =>
                        requiredProcessModel.job_process.id === jobProcess.id && requiredProcessModel.side === side);
                    if(requiredProcess) {
                        setChecked((prevState => {
                            prevState[i][jobProcess.name] = true;
                            return prevState;
                        }))
                        const arr = requiredProcess.capabilities.map((capability) => {
                            return capability.id;
                        })
                        setCapabilitiesSelected((prevState => {
                            prevState[i][jobProcess.name] = arr;
                            return prevState;
                        }))
                    } else {
                        setChecked((prevState => {
                            prevState[i][jobProcess.name] = false;
                            return prevState;
                        }))
                        setCapabilitiesSelected((prevState => {
                            prevState[i][jobProcess.name] = [];
                            return prevState;
                        }))
                    }
                }
            })
        });
    }, [jobModel, jobProcesses])

    const checkBoxOnChange = (e, side: number, jobProcessName: string) => {
        const tmp = [...checked];
        tmp[side][jobProcessName] = e.target.checked;
        setChecked(tmp);
    }

    const selectOnChange = (e: any, value: [], side: number, jobProcessName: string) => {
        const tmp = [...capabilitiesSelected];
        tmp[side][jobProcessName] = value;
        setCapabilitiesSelected(tmp);
    }

    function setAllOnSide(side: number, value: boolean){
        const tmp = [...checked];

        for(let processName in tmp[side]){
            tmp[side][processName] = value;
        }
        setChecked(tmp);
    }

    const headers = (side: string) => {
        let header;
        let button_side;

        if(side === 'left') {
            header = t`Left side`;
            button_side = 0;
        } else {
            header = t`Right side`;
            button_side = 1;
        }

        return (
            <div>
                <Typography variant={'h6'} className={classes.requiredProcessSide}>
                    {header}
                </Typography>
                <Grid container justifyContent="flex-start">
                    <Grid item>
                        <Button onClick={()=>{setAllOnSide(button_side, true)}} variant={'contained'} className={classes.addAll}>
                            Add All
                        </Button>
                    </Grid>
                    <Grid item>
                        <Button onClick={()=>{setAllOnSide(button_side, false)}} variant={'contained'} className={classes.removeAll}>
                            Remove All
                        </Button>
                    </Grid>
                </Grid>
            </div>
)
    }


    return (
        <Dialog open={open} onClose={handleClose} aria-labelledby="form-dialog-title" fullWidth={true} maxWidth={'md'}>
            <DialogTitle id="form-dialog-title">{t`Add a required process`}</DialogTitle>
            <DialogContent>
                <Grid container item key={'main-grid'} xs={12}>
                    <Grid item xs={12}>
                        <Grid container direction='column'>
                            <Grid item xs={12}>
                                <Grid container item xs={12}>
                                    <Grid item xs={6}>
                                        {displayRightLensFirst ?
                                            headers('right') :
                                            headers('left')
                                        }
                                    </Grid>
                                    <Grid item xs={6}>
                                        {displayRightLensFirst ?
                                            headers('left') :
                                            headers('right')
                                        }
                                    </Grid>
                                </Grid>
                            </Grid>
                            <RequiredProcessCheckboxFactory checkBoxOnChange={checkBoxOnChange} checked={checked} capabilitiesSelected={capabilitiesSelected} selectOnChange={selectOnChange} jobProcesses={fixedJobProcesses}/>
                        </Grid>
                    </Grid>
                </Grid>
                <DialogActions>
                    <Button onClick={handleClose} color={"secondary"}>
                        close
                    </Button>
                    <Button onClick={e => handleSubmit(checked, capabilitiesSelected)} color={"secondary"}>
                        Submit
                    </Button>
                </DialogActions>
            </DialogContent>
        </Dialog>
    )
});