import * as React from "react";
import {observer} from "mobx-react-lite";
import {t} from "@lingui/macro";
import {TabPanel, TabPanelProps} from "../generic/TabPanel";
import {Conditional, ModelCheckbox, ModelInput, ModelSelect} from "@rx-suite/ui";
import {createStyles, Divider, LinearProgress, Theme, Typography} from "@material-ui/core";
import {makeStyles} from "@material-ui/core/styles";
import {JobDataLmsTable} from "./LmsConfig/JobDataLmsTable";
import {HintList} from "../generic/HintList";
import {SettingStore} from "../../stores/SettingStore";

type JobDataSettingsTabProps = {
    disable: boolean;
} & TabPanelProps

const useStyles = makeStyles((theme: Theme) =>
    createStyles({
        divider: {
            marginTop: theme.spacing(1),
            marginBottom: theme.spacing(1)
        },
        bold: {
            fontWeight: 'bold'
        },
    })
);

export const JobDataSettingsTab: React.FunctionComponent<JobDataSettingsTabProps> = observer((props) => {
    const {value, index, disable} = props;

    const settings = SettingStore.selectedModel;

    const classes = useStyles();

    if(!settings){
        return <LinearProgress/>
    }

    return (
        <TabPanel value={value} index={index}>
            <ModelSelect name={'tag_data_source'} options={dataSourceOptions}/>
            <Conditional condition={settings.viewModel.tagSourceIsLms}>
                <ModelSelect
                    name={'lms_rule_mode'}
                    label={t`Call LMS based on`}
                    options={lmsRuleOptions}
                    hint={
                        {
                            title: 'LMS Rule Mode',
                            children: <HintList
                                primaryText={t`Determines which LMS is called first when multiple are configured.`}
                                items={lmsPriorityHint}
                            />
                        }
                    }
                />
                <JobDataLmsTable/>
                <div>
                    <ModelCheckbox
                        name={ 'post_job_data'}
                        label={t`Post job data to Rx-Data?`}
                        disabled={disable}
                        hint={
                            {
                                title: 'Post to Rx-Data',
                                text: 'When this setting is enabled, the tag data retrieved from the LMS is saved to Rx-Data.'
                            }
                        }
                    />
                </div>
            </Conditional>
            <Conditional condition={settings.viewModel.tagSourceIsRxData}>
                <ModelInput name={'rx_data_max_fetch_attempts'} label={t`Rx-Data Max Fetch Attempts`}/>
                <ModelInput name={'rx_data_fetch_timeout'} label={t`Rx-Data Fetch Timeout`}/>
            </Conditional>
            <div>
                <ModelCheckbox
                    name={'use_default_processes_on_fail_fetch'}
                    label={t`Use Default Processes on Fail Fetch?`}
                    disabled={disable}
                />
            </div>
            <Divider className={classes.divider}/>
            <Typography variant={'h5'}>{t`Other Job Settings`}</Typography>
            <ModelSelect name={'job_stale_time'} label={t`Remove Inactive Jobs After`} options={jobStaleOptions}/>
        </TabPanel>
    );
});

const lmsPriorityHint = [
    {title: `LMS Priority`, text: t`Try to find the job in each LMS in order of priority. Use this mode when there is a single LMS.`},
    {title: `Job Number Length`, text: t`Use the Job Number's length to determine which LMS to call.`}
];

const lmsRuleOptions = [
    {label: 'LMS Priority', value: 'priority_based'},
    {label: 'Job Number Length', value: 'length_based'}
];

const jobStaleOptions = [
    {label: '12 hours', value: 43200},
    {label: '1 day', value: 86400},
    {label: '2 days', value: 172800},
    {label: '3 days', value: 259200},
    {label: '5 days', value: 432000},
    {label: '10 days', value: 864000}
];

const dataSourceOptions = [
    {label: 'LMS', value: 'lms'},
    {label: 'Rx-Data', value: 'rx-data'},
    {label: 'none', value: 'none'}
]


