import * as React from "react";
import {observer} from "mobx-react-lite";
import {t, Trans} from "@lingui/macro";
import {SettingModel} from "../../stores/SettingStore";
import {TabPanel, TabPanelProps} from "../generic/TabPanel";
import {CheckboxField, ModelCheckbox, ModelInput, ModelSelect} from "@rx-suite/ui";
import {makeStyles} from "@material-ui/core/styles";
import {createStyles, Theme} from "@material-ui/core";

type RoutingSettingsTabProps = {
    settings: SettingModel
    disable: boolean;
} & TabPanelProps

const loadBalancingOptions = () => {
    return [
        {label: 'None', value: 'none'},
        {label: 'Soft Balancing', value: 'soft'},
        {label: 'Hard Balancing', value: 'hard'}
    ]
}

const useStyles = makeStyles((theme: Theme) =>
    createStyles({
        bold: {
            fontWeight: 'bold'
        },
    })
);

export const RoutingSettingsTab: React.FunctionComponent<RoutingSettingsTabProps> = observer((props) => {
    const {settings, value, index, disable} = props;

    const [markDeadJobs, setMarkDeadJobs] = React.useState(settings.mark_job_dead_after_x_visit > 0);
    const handleChangeMarkDeadJobs = (e) => {
        if(!e.target.checked){
            settings.viewModel.mark_job_dead_after_x_visit = -1;
        }else{
            if(settings.viewModel.mark_job_dead_after_x_visit === -1){
                settings.viewModel.mark_job_dead_after_x_visit = 10;
            }
        }
        setMarkDeadJobs(e.target.checked)
    }

    const classes = useStyles();

    return (
        <TabPanel value={value} index={index}>
            <ModelInput name={'routing_level'} disabled={true}/>
            <ModelSelect name={'load_balancing_mode'} options={loadBalancingOptions()} hint={{title: 'Load Balancing Options', children:
                    <div>
                        <div><Trans><span className={classes.bold}>Soft Balancing:</span> Increase load score when a job cannot be routed to a machine due to a blocked path. (recommended)</Trans></div>
                        <div><Trans><span className={classes.bold}>Hard Balancing:</span> Only increase load score when a job is physically routed to a machine.</Trans></div>
                    </div>
            }}/>
            <div>
                <CheckboxField
                    label={t`Reject Dead Jobs`}
                    checked={markDeadJobs}
                    disabled={disable}
                    onChange={handleChangeMarkDeadJobs}
                    hint={
                        {
                            title: t`Reject Dead Jobs`,
                            text: t`Reject jobs that have visited the same decision point too many time. This setting can be used to catch jobs stuck in a loop.`
                        }
                    }
                />
            </div>
            <ModelInput
                label={t`Mark Dead After X Visits`}
                name={'mark_job_dead_after_x_visit'}
                hidden={!markDeadJobs}
            />
            <div>
                <ModelCheckbox
                    name={'route_cache_enabled'}
                    hint={
                        {
                            title: t`Enable Route Cache`,
                            children: <div>
                                <div>
                                    <Trans>When enabled, the router will cache possible paths at every decision point, reducing the time it takes to make a routing decision.</Trans>
                                </div>
                                <div>
                                    <span className={classes.bold}>
                                        <Trans>Warning: enabling this setting can severely increase memory usage and should only be considered for large conveyor lines.</Trans>
                                    </span>
                                </div>
                            </div>
                        }
                    }
                />
            </div>
            <div>
                <ModelCheckbox
                    name={'loop_clustering_enabled'}
                    hint={
                        {
                            title: t`Enable Loop Clustering`,
                            children: <div>
                                <div>
                                    <Trans>When enabled, the router will cache possible paths at every decision point, reducing the time it takes to make a routing decision.</Trans>
                                </div>
                                <div>
                                    <span className={classes.bold}>
                                        <Trans>Warning: Experimental Feature. Enabling this may result in unexpected routing behaviors.</Trans>
                                    </span>
                                </div>
                            </div>
                        }
                    }
                />
            </div>
        </TabPanel>
    );
});
