import React, {useEffect} from "react";
import {
    CheckboxField,
    FormProps,
    ModelForm,
    ModelInput,
    ModelTableDefaultActions
} from "@rx-suite/ui";
import {observer} from "mobx-react-lite";
import { t, Trans } from "@lingui/macro"
import {Accordion, AccordionDetails, AccordionSummary, Button, Typography} from "@material-ui/core";
import {useSnackbar} from "notistack";
import {ConnectorModel, ConnectorStore} from "../../stores/Device/ConnectorStore";
import ReactTable from "react-table";
import {Model} from "@rx-suite/data";
import {DeviceTimerModel, DeviceTimerStore} from "../../stores/Device/DeviceTimerStore";
import {RoutingCommandModel, RoutingCommandStore} from "../../stores/Device/RoutingCommandStore";
import {RoutingCommandDialogComponent} from "../../components/device_type/RoutingCommandDialogComponent";
import {DeviceTimerDialogComponent} from "../../components/device_type/DeviceTimerDialogComponent";
import ExpandMoreIcon from "@material-ui/icons/ExpandMore";
import {SettingStore} from "../../stores/SettingStore"
import {JobRouterControlStore} from "../../stores/JobRouterControlStore";

export const DeviceTypeForm: React.FunctionComponent<FormProps> = observer(({model}) => {

    const {enqueueSnackbar} = useSnackbar();

    const settings = SettingStore.selectedModel
    const routerInfo = JobRouterControlStore.selectedModel;

    const [expanded, setExpanded] = React.useState<string | false>(false);
    const [routingCommandOpen, setRoutingCommandOpen] = React.useState(false);
    const [deviceTimerOpen, setDeviceTimerOpen] = React.useState(false);

    const [routingCommandModels, setRoutingCommandModels] = React.useState<RoutingCommandModel[]>([])
    const [deviceTimerModels, setDeviceTimerModels] = React.useState<DeviceTimerModel[]>([])
    const [connectorModels, setConnectorModels] = React.useState<ConnectorModel[]>([])

    useEffect(() => {
        const resp = RoutingCommandStore.getByIds(model.device_routing_command_ids);
        resp.then((data) => {
          setRoutingCommandModels(data);
        });
    }, [model.device_routing_command_ids])
    const routingCommandModel = RoutingCommandStore.createModel();

    useEffect(() => {
        const resp = DeviceTimerStore.getByIds(model.device_timer_ids);
        resp.then((data) => {
          setDeviceTimerModels(data);
        });
    }, [model.device_timer_ids])
    const deviceTimerModel = DeviceTimerStore.createModel();

    useEffect(() => {
        const resp = ConnectorStore.getByIds(model.opcua_connector_ids);
        resp.then((data) => {
          setConnectorModels(data);
        });
    }, [model.connector_ids])

    const connectorColumns = [{Header: t`Connectors`, columns: [
        {Header: t`Name`, accessor: 'name'},
        {Header: t`Variable`, accessor: 'variable'},
        {Header: t`Type`, accessor: 'connector_type'},
        {Header: t`Connector Priority`, accessor: 'priority'},
        {Header: t`Action delay (milliseconds)`, accessor: 'action_delay'},
        {accessor: d => d, id: '_actions', style: {textAlign: "right"}, Cell: (cellInfo: { value: Model }) => {
            if (!cellInfo.value || !cellInfo.value.getId || !(routerInfo?.is_offline)) {
                return null;
            }
            return <>
                <ModelTableDefaultActions store={ConnectorStore} model={cellInfo.value}/>
            </>}}]}]

    const routingCommandColumns = [{Header: t`Routing Commands`, columns: [
        {Header: t`Command`, accessor: 'command'},
        {Header: t`Parameter`, accessor: 'parameter'},
        {Header: t`Connectors`, id: 'connectors', accessor: data => {
            let output = [];
            data.opcua_connector_ids.forEach((connectorId) => {
                if (connectorModels.length === 0) { return; }
                output.push(connectorModels.find((connectorModel) => connectorModel.id === connectorId));
            })
            return output.map((model) => model.name).join(', ');
            }},
        {accessor: d => d, id: '_actions', style: {textAlign: "right"}, Cell: (cellInfo: { value: Model }) => {
            if (!cellInfo.value || !cellInfo.value.getId || !(routerInfo?.is_offline)) {
                return null;
            }
            return <>
                <ModelTableDefaultActions store={RoutingCommandStore} model={cellInfo.value}/>
            </>}}
        ]}]


    const deviceTimerColumns = [{Header: t`Routing Commands`, columns: [
        {Header: t`Variable`, accessor: 'variable'},
        {Header: t`Time (milliseconds)`, accessor: 'time'},
        {Header: t`Description`, accessor: 'description'},
        {accessor: d => d, id: '_actions', style: {textAlign: "right"}, Cell: (cellInfo: { value: Model }) => {
            if (!cellInfo.value || !cellInfo.value.getId || !(routerInfo?.is_offline)) {
                return null;
            }
            return <>
                <ModelTableDefaultActions store={DeviceTimerStore} model={cellInfo.value} />
            </>}}]
    }]

    const handleRoutingCommandClickOpen = () => {
        setRoutingCommandOpen(true);
    }

    const handleRoutingCommandClose = () => {
        setRoutingCommandOpen(false);
    }

    const handleDeviceTimerClickOpen = () => {
        setDeviceTimerOpen(true);
    }

    const handleDeviceTimerClickClose = () => {
        setDeviceTimerOpen(false);
    }

    const handleRoutingCommandSubmit = (submitModel: Model) => {
        submitModel.viewModel.device_type_id = model.id;
        submitModel.viewModel.submit();
        submitModel.commit().then(function(resp:any){
            enqueueSnackbar(`${t`Routing command successfully created`}`, {variant: "success"})
            setRoutingCommandOpen(false);
            location.reload()
        });
    }

    const handleDeviceTimerSubmit = (submitModel: Model) => {
        submitModel.viewModel.device_type_id = model.id;
        submitModel.viewModel.submit();
        submitModel.commit().then(function(resp:any){
            enqueueSnackbar(`${t`Device timer successfully created`}`, {variant: "success"})
            setDeviceTimerOpen(false);
        });
    }

    const handleAccordionChange = (panel: string) => (event: React.ChangeEvent<{}>, isExpanded: boolean) => {
        setExpanded(isExpanded ? panel : false);
    };

    return (
        <ModelForm model={model} suppressMakeCopy suppressImportExport>
            <Typography variant={'h4'}>
                <Trans>
                    Device type configuration
                </Trans>
            </Typography>
            <ModelInput name={'name'} disabled={!model.isNew}/>
            {settings?.variant === 'modular' && <ModelInput name={'root_node_id'} disabled={!(routerInfo?.is_offline)}/>}
            <ModelInput name={'routing_retry_delay'}/>
            <ModelInput name={'delay_on_command_complete'}/>
            <CheckboxField name={'all_in_one'} color={"secondary"} label={t`All in one`} onChange={model.handleCheckboxChange} checked={model.viewModel.all_in_one} disabled={!model.isNew}/>
            <br/>
            {!model.isNew &&
                <Accordion expanded={expanded === 'panel1'} onChange={handleAccordionChange('panel1')}>
                    <AccordionSummary expandIcon={<ExpandMoreIcon/>} aria-controls="panel1bh-content" id="panel1bh-header">
                        <Typography variant={'h6'}>
                            <Trans>
                                Routing Commands
                            </Trans>
                        </Typography>
                    </AccordionSummary>
                    <AccordionDetails>
                        <div style={{width: '100%'}}>
                            {routerInfo?.is_offline &&
                            <Button variant="outlined" color="secondary" onClick={handleRoutingCommandClickOpen}>
                                Add Routing command
                            </Button>}
                            <br/>
                            <ReactTable
                                data={routingCommandModels}
                                columns={routingCommandColumns}
                                minRows={0}
                                defaultPageSize={10}
                            />
                            {open &&
                                <RoutingCommandDialogComponent model={routingCommandModel} open={routingCommandOpen} handleSubmit={handleRoutingCommandSubmit} handleClose={handleRoutingCommandClose}/>}
                        </div>
                    </AccordionDetails>
                </Accordion>
            }
            {!model.isNew &&
                <Accordion expanded={expanded === 'panel2'} onChange={handleAccordionChange('panel2')}>
                    <AccordionSummary expandIcon={<ExpandMoreIcon/>} aria-controls="panel2bh-content" id="panel2bh-header">
                        <Typography variant={'h6'}>
                            <Trans>
                                Connectors
                            </Trans>
                        </Typography>
                    </AccordionSummary>
                    <AccordionDetails>
                        <div style={{width: '100%'}}>
                            <ReactTable
                                data={connectorModels}
                                columns={connectorColumns}
                                minRows={0}
                                defaultPageSize={10}
                            />
                        </div>
                    </AccordionDetails>
                </Accordion>
            }
            {!model.isNew &&
                <Accordion expanded={expanded === 'panel3'} onChange={handleAccordionChange('panel3')}>
                    <AccordionSummary expandIcon={<ExpandMoreIcon/>} aria-controls="panel3bh-content" id="panel3bh-header">
                        <Typography variant={'h6'}>
                            <Trans>
                                Timers
                            </Trans>
                        </Typography>
                    </AccordionSummary>
                    <AccordionDetails>
                        <div style={{width: '100%'}}>
                            {routerInfo?.is_offline &&
                            <Button variant="outlined" color="secondary" onClick={handleDeviceTimerClickOpen}>
                                Add a Timer
                            </Button>}
                            <br/>
                            <ReactTable
                                data={deviceTimerModels}
                                columns={deviceTimerColumns}
                                minRows={0}
                                defaultPageSize={10}
                            />
                            {open &&
                                <DeviceTimerDialogComponent model={deviceTimerModel} open={deviceTimerOpen}
                                                        handleSubmit={handleDeviceTimerSubmit}
                                                        handleClose={handleDeviceTimerClickClose}/>}
                        </div>
                    </AccordionDetails>
                </Accordion>
            }
        </ModelForm>
    )})