import React from "react";
import {
    FormProps,
    makeColumn,
    ModelForm,
    ModelInput,
    ModelSelect,
} from "@rx-suite/ui";
import {observer} from "mobx-react-lite";
import { t, Trans } from "@lingui/macro"
import {Button, Grid, Typography} from "@material-ui/core";
import {useSnackbar} from "notistack";
import {CapabilitiesModel, CapabilityStore} from "../../stores/machine/CapabilityStore";
import {MachineModel, MachineStore} from "../../stores/machine/MachineStore";

export const RequiredProcessForm: React.FunctionComponent<FormProps> = observer(({model}) => {

    const {enqueueSnackbar} = useSnackbar();
    const capabilities = CapabilityStore.getAll();
    const machines = MachineStore.getAll();

    const getPossibleCapabilitiesOptions = () => {
        const possibleCapabilities = capabilities.filter((capability) => {return capability.job_process_id === model.job_process.id})
        return possibleCapabilities.map((capability: CapabilitiesModel) => ({label: capability.resolvedName, value: capability.id}))
    }

    const getCapabilityValues = () => {
        const possibleCapabilities = getPossibleCapabilitiesOptions();
        const selected = [];
        for(let i =0; i < possibleCapabilities.length; i++) {
            if (model.viewModel.capability_ids) {
                if (model.viewModel.capability_ids.find((capability_id) => {
                        return (capability_id === possibleCapabilities[i].value)
                    }))
                {
                    selected.push(possibleCapabilities[i].value);
                }
            }
        }
        return selected
    }

    const getPossibleMachineOptions = () => {
        if(model) {
            let possibleMachines = machines.filter((machine) => {return machine.job_process_id === model.job_process.id}) as MachineModel[]
            let capabilityIds = new Set(model.capability_ids as number[])
            if(model.opposite_process) {
                capabilityIds = new Set([...capabilityIds, ...model.opposite_process.capability_ids])
            }
            for(let capabilityId  of capabilityIds) {
                possibleMachines = possibleMachines.filter((machine: MachineModel) => machine.has_capability(capabilityId))
            }
            return possibleMachines.map((machine: MachineModel) => ({label: machine.name, value: machine.name}))
        }
    }

    const completeProcessOnClick = React.useCallback(async () => {
        model.viewModel.submit();
        try {
            const resp = await fetch(`/api/required_processes/${model.id}/complete`, {
                method: "POST",
                headers: {'Content-Type': 'application/json'},
                body: JSON.stringify({required_process: model})
            })
            const data = await resp.json();
            if(data.error) {
                enqueueSnackbar(data.error, {variant: "error"});
            } else {
                enqueueSnackbar(`${t`Required process successfully updated`}`, {variant: "success"})
            }
        } catch (err) {
            console.error(err);
            enqueueSnackbar({err}, {variant: "error"});
        }
    }, [])

    const resetProcessOnClick = React.useCallback(async () => {
        model.viewModel.submit();
        try {
            const resp = await fetch(`/api/required_processes/${model.id}/reset`, {
                method: "POST",
                headers: {'Content-Type': 'application/json'},
                body: JSON.stringify({required_process: model})
            })
            const data = await resp.json();
            if(data.error) {
                enqueueSnackbar(data.error, {variant: "error"});
            } else {
                enqueueSnackbar(`${t`Required process successfully updated`}`, {variant: "success"})
            }
        } catch (err) {
            console.error(err);
            enqueueSnackbar({err}, {variant: "error"});
        }
    }, [])

    const completeBothProcessOnClick = React.useCallback(async () => {
        model.viewModel.submit();
        try {
            const resp = await fetch(`/api/required_processes/${model.id}/complete_both`, {
                method: "POST",
                headers: {'Content-Type': 'application/json'},
                body: JSON.stringify({required_process: model})
            })
            const data = await resp.json();
            if(data.error) {
                enqueueSnackbar(data.error, {variant: "error"});
            } else {
                enqueueSnackbar(`${t`Required process successfully updated`}`, {variant: "success"})
            }
        } catch (err) {
            console.error(err);
            enqueueSnackbar({err}, {variant: "error"});
        }
    }, [])

    return(
        <ModelForm model={model}>
            <br/>
            <Typography variant={'h4'}>
                <Trans>
                    Process - {model.side.charAt(0).toUpperCase() + model.side.slice(1)} lens: {model.job_process?.resolvedName}
                </Trans>
            </Typography>
            <ModelInput name={'state'} disabled={true}/>
            <ModelSelect name={'capability_ids'} options={getPossibleCapabilitiesOptions()} multiple={true} values={getCapabilityValues()} disabled={model.state === 'FINISHED'}/>
            <ModelSelect name={'force_to_machine'} options={getPossibleMachineOptions()} disabled={model.state === 'FINISHED'}/>
            <Grid container>
                <Grid item xs={4}>
                    <Button variant={'contained'}  color={'secondary'} style={{width: '90%', paddingLeft: '100'}} onClick={completeProcessOnClick}>
                        {t`Complete process`}
                    </Button>
                </Grid>
                <Grid item xs={4}>
                    <Button variant={'contained'} color={'secondary'} style={{width: '90%'}} onClick={resetProcessOnClick}>
                        {t`Reset process`}
                    </Button>
                </Grid>
                <Grid item xs={4}>
                    <Button variant={'contained'} color={'secondary'} style={{width: '90%'}} onClick={completeBothProcessOnClick}>
                        {t`Complete process for both sides`}
                    </Button>
                </Grid>
            </Grid>
        </ModelForm>
    )
});