import * as React from 'react';
import { TrayWidget } from './TrayWidget';
import { TrayItemWidget } from './TrayItemWidget';
import {BasePositionModelOptions, CanvasWidget} from '@projectstorm/react-canvas-core';
import { DemoCanvasWidget } from './DemoCanvasWidget';
import styled from '@emotion/styled';
import {
	ExpansionPanel,
	ExpansionPanelDetails,
	ExpansionPanelSummary,
	LinearProgress,
	makeStyles,
	Typography
} from "@material-ui/core";
import {observer, useForceUpdate} from "mobx-react-lite";
import {Model} from "@rx-suite/data";
import {FormProps, FullscreenButton, LinkButton, ModelForm, ModelInput} from "@rx-suite/ui";
import {JobModel} from "../stores/JobStore";
import {useEffect, useRef, useState} from "react";
import {ConnectorModel} from "../stores/Device/ConnectorStore";
import {RouteApplication} from "../pages/routes/RouteApplication";
import ExpandMoreIcon from "@material-ui/icons/ExpandMore";
import {t} from "@lingui/macro";
import {DemoWorkspaceWidget} from "./DemoWorkspaceWidget";
import {CapabilityStore} from "../stores/machine/CapabilityStore";
import {RoutingGraphModel, RoutingGraphStore} from "../stores/RoutingGraphStore";
import {useSnackbar} from "notistack";
import {Edit} from "@material-ui/icons";
import {ComponentNodeModel} from "./ComponentNode/ComponentNodeModel";
import {SettingModel, SettingStore} from "../stores/SettingStore";
import {ComponentStore} from "../stores/ComponentStore";

export const Body = styled.div`
	flex-grow: 1;
	display: flex;
	flex-direction: column;
`;

export const Header = styled.div`
	display: flex;
	background: rgb(30, 30, 30);
	flex-grow: 0;
	flex-shrink: 0;
	color: white;
	font-family: Helvetica, Arial, sans-serif;
	padding: 10px;
	align-items: center;
`;

export const Content = styled.div`
	display: flex;
	flex-grow: 1;
	height: 90vh
`;

export const Layer = styled.div`
	position: relative;
	flex-grow: 1;
`;

export const SubmitButton = styled.button`
	background: rgb(255, 153, 0);
	font-size: 14px;
	padding: 5px 10px;
	border: none;
	color: black;
	outline: none;
	cursor: pointer;
	margin: 2px;
	border-radius: 3px;
	&:hover {
		background: rgb(178, 107, 0);
		color: rgb(64, 169, 255);
	}
`;

const useStyle = makeStyles(theme => ({
	myCanvas: {
		"& svg" : {
			overflow: 'visible'
		}
	},
	fullscreenLayout: {
        "&:fullscreen": {
            height: '100vw',
            width: '100vh',
            overflow: 'auto',
            padding: theme.spacing(2),
        }
    }
}));

interface BodyWidgetComponentProps {
	app: RouteApplication,
    preProcessedNodes: {
		id: number, name: string, inputs: ConnectorModel[], outputs: ConnectorModel[], type: string}[],
	routingGraph: RoutingGraphModel,
	//settings: SettingModel
}

export const BodyWidgetComponent: React.FunctionComponent<BodyWidgetComponentProps> = observer(({app, preProcessedNodes, routingGraph, /*settings*/}) => {

	const {enqueueSnackbar} = useSnackbar();
	const classes = useStyle();
	const [trayNodes, setTrayNodes] = useState<ComponentNodeModel[]>([]);
	const [activeNodes, setActiveNodes] = useState<ComponentNodeModel[]>([]);

	const activeNodesRef = useRef(activeNodes);
	activeNodesRef.current = activeNodes;

	const trayNodesRef = useRef(trayNodes);
	trayNodesRef.current = trayNodes;

	const fullscreenLayoutRef = React.createRef<HTMLDivElement>();

	const listener = {
		entityRemoved: (event) => {
			const tmpActiveNodes = activeNodesRef.current
			const index = tmpActiveNodes.findIndex((nodeModel) => nodeModel.options.id === event.entity.options.id);
			const trayNode = tmpActiveNodes.splice(index, 1)[0];
			const component = ComponentStore.models.find((component) => component.id === trayNode.getOptions().id);
			trayNode.options.name = component ? component.name : '';
			const tmp = [...trayNodesRef.current, trayNode].sort((a,b) => (a.options.id - b.options.id));
			setTrayNodes(tmp);
			setActiveNodes(tmpActiveNodes);
			trayNode.clearListeners();
		}
	};

	useEffect(() => {
		const nodes = [];
		const activeNodeModels:ComponentNodeModel[] = Object.values(app.getActiveDiagram().getActiveNodeLayer().getModels());
		const loadedActiveNodes: ComponentNodeModel[] = [];
		preProcessedNodes = preProcessedNodes.filter((preProcessedNode) => {
			return !(activeNodeModels.some((activeNodeModel) => {
				const inPorts = activeNodeModel.getInPorts();
				if (activeNodeModel.options.id !== preProcessedNode.id)
					return false;
				/*if (inPorts.length !== preProcessedNode.inputs.length)
					return false
				for(let i = 0; i < preProcessedNode.inputs.length; i++) {
					for (let j = 0; j < inPorts.length; j++) {
						if (preProcessedNode.inputs[i].name === inPorts[j].getOptions().name) {
							console.log('something')
						}
						else {
							console.log('something else entirely')
						}
					}
				}
				if(activeNodeModel.getInPorts().every((inPort) => preProcessedNode.inputs.some((input) => inPort.getOptions().name === input.name)))
					return false*/
				activeNodeModel.registerListener(listener);
				loadedActiveNodes.push(activeNodeModel);
				return true
			}));
		})
		setActiveNodes(loadedActiveNodes);

		for(let i = 0; i < preProcessedNodes.length; i++) {
			const node = preProcessedNodes[i];
			const nodeModel = new ComponentNodeModel({id: node.id, color: node.type === 'device' ? 'rgb(255,153,0)': 'rgb(20,110,180)', name: node.name})
			node.inputs.forEach((input) => {
				nodeModel.addInPort(input.name);
			});
			if(node.outputs.length > 0) {
				node.outputs.forEach((output) => {
					nodeModel.addOutPort(output.name);
				});
			}
			else {
				nodeModel.addOutPort(node.inputs[0].name + ' (1)');
			}
			nodes.push(nodeModel);
		}

		setTrayNodes(nodes.sort((a,b) => (a.options.id - b.options.id)));
		}, [])

	const nodesTrayItems = trayNodesRef.current.map((nodeModel, key) => {
		const options = nodeModel.getOptions();
		return <TrayItemWidget model={{ type: options.id }} id={options.id} color={options.color} name={options.name}/>
	})

	const handleRoutingGraphSubmit = (event) => {
		const model = app.getActiveDiagram();
		routingGraph.viewModel.serialized_data = JSON.stringify(model.serialize());
		console.log(routingGraph.viewModel.serialized_data);
		routingGraph.viewModel.submit();
        routingGraph.commit().then(function(resp:any) {
            enqueueSnackbar(`${t`Routing Graph successfully updated`}`, {variant: "success"});
        });
	}

	const buttons = () => {
		return (
			<React.Fragment>
				<FullscreenButton fullscreenRef={fullscreenLayoutRef} style={{color: '#fafafa', backgroundColor: 'rgba(146, 146, 146, 0.56)'}}/>
				{/*settings?.router_status === 'offline' &&*/
					<SubmitButton onClick={handleRoutingGraphSubmit}>Submit</SubmitButton>}
			</React.Fragment>
		)
	}

	return (
		<div className={classes.fullscreenLayout} ref={fullscreenLayoutRef}>
			<Body>
				<Header>
					<div className="title">Routing graph builder</div>
				</Header>
				<Content>
					<TrayWidget>
						{nodesTrayItems}
					</TrayWidget>
					<Layer
						onDrop={(event) => {
							let data = null
							try {
								data = JSON.parse(event.dataTransfer.getData('storm-diagram-node'));
							}
							catch (e) {
								return;
							}
							const nodeModel = trayNodes.find((nodeModel) => nodeModel.options.id === data.id);

							const point = app.getDiagramEngine().getRelativeMousePoint(event);
							nodeModel.setPosition(point);
							app.getDiagramEngine().getModel().addNode(nodeModel);

							const tmpTrayNodes = trayNodesRef.current;
							const index = tmpTrayNodes.findIndex((nodeModel) => nodeModel.options.id === data.id);
							const activeNode = tmpTrayNodes.splice(index, 1)[0];
							setTrayNodes(tmpTrayNodes);
							setActiveNodes(activeNodes.concat(activeNode));

							nodeModel.registerListener(listener);
						}}
						onDragOver={(event) => {
							event.preventDefault();
						}}>
						<DemoWorkspaceWidget buttons={buttons()}>
							<DemoCanvasWidget>
								<CanvasWidget engine={app.getDiagramEngine()} className={classes.myCanvas}/>
							</DemoCanvasWidget>
						</DemoWorkspaceWidget>
					</Layer>
				</Content>
			</Body>
		</div>
	)
})