import { NodeModel, NodeModelGenerics, DiagramEngine, LinkModel, RightAngleLinkModel } from "@projectstorm/react-diagrams";
import {BasePositionModelOptions, DeserializeEvent} from '@projectstorm/react-canvas-core';
import {AbstractModelFactory} from '@projectstorm/react-canvas-core'
import {DefaultPortModel} from '@projectstorm/react-diagrams';
import {PortModelAlignment} from "@projectstorm/react-diagrams-core";
import {RightAnglePortModel} from "./RightAnglePortModel";

export interface ComponentNodeModelOptions extends BasePositionModelOptions {
	name?: string;
	color?: string;
}

export interface ComponentNodeModelGenerics extends NodeModelGenerics {
	id: number;
	OPTIONS: ComponentNodeModelOptions;
}

export class ComponentNodeModel extends NodeModel<ComponentNodeModelGenerics> {
	protected portsIn: RightAnglePortModel[];
	protected portsOut: RightAnglePortModel[];
	protected id: number;

	constructor(id: number, color: string);
	constructor(options?: ComponentNodeModelOptions);
	constructor(options: any = {}, color?: string) {
		if (typeof options === 'number') {
			options = {
				id: options,
				name: 'untitled',
				color: color
			};
		}
		super({
			type: 'component',
			color: 'rgb(0,192,255)',
			...options
		});
		this.portsIn = [];
		this.portsOut = [];
		this.id = options.id;
	}

	reverse(engine) {
	    const ports = this.getPorts();
	    for (const [key, port] of Object.entries(ports)) {
            const serialized_data = port.serialize();
            if(serialized_data.alignment === PortModelAlignment.LEFT) {
            	serialized_data.alignment = PortModelAlignment.RIGHT;
			} else if(serialized_data.alignment === PortModelAlignment.RIGHT) {
            	serialized_data.alignment = PortModelAlignment.LEFT;
			}
            port.deserialize({data: serialized_data, engine: engine})
        }
	    engine.repaintCanvas();
    }

    updatePortLists(leftPortList: DefaultPortModel[], rightPortList: DefaultPortModel[]) {
		const firstLeftPort = leftPortList[0].port;
		const inPortList = firstLeftPort.getOptions().in ? leftPortList : rightPortList;
		const outPortList = firstLeftPort.getOptions().in ? rightPortList : leftPortList;

		this.portsIn = inPortList.map((listPort) => listPort.port);
		this.portsOut = outPortList.map((listPort) => listPort.port);
	}

    addPort<T extends DefaultPortModel>(port: T): T {
		/*if(!(port instanceof RightAnglePortModel)) {
			port = new RightAnglePortModel({
				in: port.getOptions().in,
				name: port.getOptions().name,
				label: port.getOptions().label,
				alignment: port.getOptions().alignment,
			});
		}*/
		super.addPort(port);
		if (port.getOptions().in) {
			if (this.portsIn.indexOf(port) === -1) {
				this.portsIn.push(port);
			}
		} else {
			if (this.portsOut.indexOf(port) === -1) {
				this.portsOut.push(port);
			}
		}
		return port;
	}

    addInPort(label: string, alignment?: PortModelAlignment, after = true): RightAnglePortModel {
		const p = new RightAnglePortModel({
			name: label,
			in: true,
			label: label,
			alignment: alignment ? alignment : PortModelAlignment.LEFT,
		});
		if (!after) {
			this.portsIn.splice(0, 0, p);
		}
		return this.addPort(p);
	}

	addOutPort(label: string, alignment?: PortModelAlignment, after = true): RightAnglePortModel {
		const p = new RightAnglePortModel({
			name: label,
			in: false,
			label: label,
			alignment: alignment ? alignment : PortModelAlignment.RIGHT,
		});
		if (!after) {
			this.portsOut.splice(0, 0, p);
		}
		return this.addPort(p);
	}

    getInPorts(): DefaultPortModel[] {
		return this.portsIn;
	}

	getOutPorts(): DefaultPortModel[] {
		return this.portsOut;
	}

	getLeftPorts(): DefaultPortModel[] {
		return this.portsIn[0].getOptions().alignment === 'left' ? this.portsIn : this.portsOut
	}

	getRightPorts(): DefaultPortModel[] {
		return this.portsIn[0].getOptions().alignment === 'right' ? this.portsIn : this.portsOut
	}

	deserialize(event: DeserializeEvent<this>) {
		super.deserialize(event);
		this.options.id = event.data.id;
		this.options.color = event.data.color;
		this.portsIn = _.map(event.data.portsInOrder, (id) => {
			console.log(this.getPorts())
			return this.getPortFromID(id);
		}) as RightAnglePortModel[];
		this.portsOut = _.map(event.data.portsOutOrder, (id) => {
			return this.getPortFromID(id);
		}) as RightAnglePortModel[];
	}

	serialize(): any {
		return {
			...super.serialize(),
			id: this.options.id,
			color: this.options.color,
			portsInOrder: _.map(this.portsIn, (port) => {
				return port.getID();
			}),
			portsOutOrder: _.map(this.portsOut, (port) => {
				return port.getID();
			})
		};
	}
}