import * as React from 'react';
import { DiagramEngine } from '@projectstorm/react-diagrams-core';
import {ComponentNodeModel, RightAnglePortModel} from "./ComponentNodeModel";
import {ComponentPortLabel} from "./ComponentPortLabelWidget";
import styled from "@emotion/styled";
import {FontAwesomeIcon} from "@fortawesome/react-fontawesome";
import {faArrowsH} from "@rx-suite/pro-solid-svg-icons";
import {IconButton} from "@material-ui/core";
import {ReactSortable} from "react-sortablejs";
import {ComponentStore} from "../../stores/ComponentStore";
import {JobRouterControlStore} from "../../stores/JobRouterControlStore";

export const Node = styled.div<{ background: string; selected: boolean }>`
    background-color: ${(p) => p.background};
    border-radius: 5px;
    font-family: sans-serif;
    color: white;
    border: solid 2px black;
    overflow: visible;
    font-size: 11px;
    border: solid 2px ${(p) => (p.selected ? 'rgb(0,192,255)' : 'black')};
`;

export const Title = styled.div`
    background: rgba(0, 0, 0, 0.3);
    display: flex;
    white-space: nowrap;
    justify-items: center;
    width: 100%
`;

export const TitleName = styled.div`
    flex-grow: 1;
    padding: 5px 5px;
`;

export const Ports = styled.div`
    display: flex;
    background-image: linear-gradient(rgba(0, 0, 0, 0.1), rgba(0, 0, 0, 0.2));
`;

export const PortsContainer = styled.ul`
	margin: 0;
	padding: 0;
	list-style: none;
    flex-grow: 1;
    display: flex;
    flex-direction: column;

    &:only-child {
        margin-right: 0px;
    }
`;

export interface ComponentNodeProps {
	node: ComponentNodeModel;
	engine: DiagramEngine;
}

interface portState {
	leftPortList: { id: string, port: RightAnglePortModel }[];
	rightPortList: { id: string, port: RightAnglePortModel }[];
}

export class ComponentNodeWidget extends React.Component<ComponentNodeProps> {

	state: portState = {
		leftPortList: [],
		rightPortList: []
	}

	private name: string;

	constructor(props) {
		super(props);
		const component = ComponentStore.models.find((component) => component.id === this.props.node.getOptions().id);
		this.name = component ? component.name as string : this.props.node.getOptions().name as string;
	}

	componentDidMount() {
		this.setState({ leftPortList: this.props.node.getLeftPorts().map((port, index) => {return {id: index, port: port}}),
							rightPortList: this.props.node.getRightPorts().map((port, index) => {return {id: index, port: port}})});
	}

	generatePort = (port: RightAnglePortModel) => {
		return <ComponentPortLabel engine={this.props.engine} port={port} key={port.getID()} />;
	};

	handlePortReverseOnClick = () => {
		this.props.node.reverse(this.props.engine);
		this.setState({leftPortList: this.state.rightPortList, rightPortList: this.state.leftPortList});
	}

	handleOnSort = () => {
		this.props.node.updatePortLists(this.state.leftPortList, this.state.rightPortList);
	}

	render() {
		return (
			<Node
				id={'Node'}
				data-default-node-name={this.props.node.getOptions().name}
				selected={this.props.node.isSelected()}
				background={this.props.node.getOptions().color}>
				<Title id={'Title'}>
					<TitleName id={'TitleName'}>{this.name}</TitleName>
					<IconButton onClick={this.handlePortReverseOnClick} style={{height: 25, width: 25}}>
						<FontAwesomeIcon icon={faArrowsH}/>
					</IconButton>
				</Title>
				<Ports>
					<ReactSortable
						list={this.state.leftPortList}
						setList={newState => this.setState({ leftPortList: newState})}
						onSort={this.handleOnSort}>
						{this.state.leftPortList.map((state) => (
							<PortsContainer>{this.generatePort(state.port)}</PortsContainer>
						))}
					</ReactSortable>
					<ReactSortable
						list={this.state.rightPortList}
						setList={newState => this.setState({ rightPortList: newState})}
						onSort={this.handleOnSort}>
						{this.state.rightPortList.map((state) => (
							<PortsContainer>{this.generatePort(state.port)}</PortsContainer>
						))}
					</ReactSortable>
				</Ports>
			</Node>
		);
	}
}