import {attribute, DataStore, Model, model} from "@rx-suite/data";
import {t} from "@lingui/macro";
import * as yup from "yup";
import {action, computed} from "mobx";

@model({
    transportLayer: "/api/lms_configs",
    permissionTarget: "Setting",
    routingInfo: {
        create: '/settings',
        read: '/settings',
        update: '/settings'
    }
})

export class LmsConfigModel extends Model {
    @attribute({
        label: t`Id`,
        validation: yup.number()
    })
    id: number = 0;

    @attribute({
        label: t`Priority`,
        validation: yup.number().positive().integer()
    })
    priority: number = 1;

    @attribute({
        label: t`Job Number Minimum Length`,
        validation: yup.number().positive().integer().nullable().notRequired()
    })
    length_left_bound: number | null = null;

    @attribute({
        label: t`Job Number Maximum Length`,
        validation: yup.number().positive().integer().nullable().notRequired()
    })
    length_right_bound: number | null = null;

    @attribute({
        label: t`LMS Type`,
        validation: yup.string()
    })
    lms_type: 'job' | 'inf' = 'job';

    get isJobLms(): boolean {
        return this.viewModel.lms_type === 'job'
    }

    get isInfLms(): boolean {
        return this.viewModel.lms_type === 'inf'
    }

    @attribute()
    online: boolean = false;

    @action
    async reinitializeConnection() {
        return await fetch(`/api/lms_configs/${this.id}/reinitialize`, {method: "POST"});
    }

    @attribute({
        label: t`LMS Connection`,
        validation: yup.number().typeError(t`Select a valid LMS connection.`)
    })
    external_dcs_id: number | null = null;

    // LEGACY ATTRIBUTES: DO NOT ALLOW EDIT - ONLY KEEP FOR VIEW PURPOSES //

    @computed
    get isUsingLegacyConnection(){
        return this.external_dcs_id == null;
    }

    @attribute({
        label: t`Name`,
        validation: yup.string().nullable()
    })
    name: string | null = null;

    @attribute({
        label: t`Ip Address`,
        validation: yup.string().nullable()
    })
    ip: string | null = null;

    @attribute({
        label: t`Port`,
        validation: yup.string().nullable()
    })
    port: string | null = null;

    @attribute({
        label: t`Device Type`,
        validation: yup.string().nullable()
    })
    device_type: string | null = null;

    @attribute({
        label: t`Delay on Socket Open`,
        validation: yup.number().nullable()
    })
    delay_on_socket_open: number | null = 0;

    @attribute({
        label: t`Hold Connection`,
        validation: yup.boolean().nullable()
    })
    hold_connection: boolean = true;

    @attribute({
        label: t`Maximum Allowed Connection Attempts`,
        validation: yup.number().nullable()
    })
    max_open_connections: number | null  = null;

    // END OF LEGACY ATTRIBUTES //

    constructor() {
        super();
        super.setup();
    }
}

export const LmsConfigStore = LmsConfigModel.store as DataStore<LmsConfigModel>;