import {attribute, DataStore, Model, model} from '@rx-suite/data';
import * as yup from 'yup';
import { t } from "@lingui/macro"
import {computed} from "mobx";

@model({
    transportLayer: "/api/settings",
    permissionTarget: "Setting",
    singleton: true,
    eager: true,
    routingInfo: {
        create: '/settings/create',
        read: '/settings/:id/edit',
        update: '/settings/:id/edit'
    }
})

export class SettingModel extends Model {
    @attribute({
        label: t`Id`,
        validation: yup.number()
    })
    name: number = 0;

    @attribute({
        label: t`Connection Type`,
        validation: yup.string().required(t`A connection type is required`)
    })
    connection_type: string = '';

    @attribute({
        label: t`Variant`,
        validation: yup.string().nullable()
    })
    variant: string = '';

    @attribute({
        label: t`Opcua server URL`,
        validation: yup.string().nullable()
    })
    opcua_server_url: string = '';

    @attribute({
        label: t`Device Root Path`,
        validation: yup.string().nullable()
    })
    device_root_path: string = '';

    @attribute({
        label: t`Use Rx-Scanners?`,
        validation: yup.string().nullable()
    })
    use_rx_scanners: boolean = false;

    @attribute({
        label: t`Use default processes if job fetcher fails?`,
        validation: yup.boolean()
    })
    use_default_processes_on_fail_fetch: boolean = false;

    @attribute({
        label: t`Mark job dead after how many visits to the same point?`,
        validation: yup.number().nullable()
    })
    mark_job_dead_after_x_visit: number = 1;

    @attribute({
        label: t`Job stale time (seconds)`,
        validation: yup.number().required(t`This field is required`)
    })
    job_stale_time: number = 0;

    @attribute({
        label: t`Launch server upon service start?`,
        validation: yup.boolean()
    })
    launch_server_on_service_start: boolean = false;

    @attribute({
        label: t`Tag data source`,
        validation: yup.string()
    })
    tag_data_source: 'lms' | 'rx-data' | 'none' = 'lms';

    @attribute({
        label: t`Max job fetcher retry`,
        validation: yup.number().nullable()
    })
    rx_data_max_fetch_attempts: number = 3;

    @attribute({
        label: t`Job fetcher timeout (seconds)`,
        validation: yup.number().nullable()
    })
    rx_data_fetch_timeout: number = 10;

    @attribute({
        label: t`Rx-data online?`,
        validation: yup.boolean()
    })
    data_online: boolean = false;

    @attribute({
        label: t`Using Rx-Data?`,
        validation: yup.boolean()
    })
    using_rx_data: boolean = false;

    @attribute({
        label: t`Loop load balancing`,
        validation: yup.string()
    })
    load_balancing_mode: string = 'none';

    @attribute({
        label: t`Data format`,
        validation: yup.string().nullable()
    })
    data_format: string = 'XML';

    @attribute({
        label: t`Post job data to Rx-Data?`,
        validation: yup.boolean().nullable()
    })
    post_job_data: boolean = false;

    @attribute({
        label: t`Routing Level`,
        validation: yup.string().nullable()
    })
    routing_level: string = 'basic';

    @attribute({
        label: t`LMS Rule Mode`,
        validation: yup.string().nullable()
    })
    lms_rule_mode: 'length_based' | 'priority_based' = 'priority_based';

    @attribute({
        label: t`Enable INF?`,
        validation: yup.boolean()
    })
    inf_enabled: boolean = false;

    @attribute({
        label: t`INF Max Attempts`,
        validation: yup.number()
    })
    inf_max_attempts: number = 3;

    @attribute({
        label: t`Time Between INF Attempts`,
        validation: yup.number()
    })
    inf_time_between_attempts: number = 60;

    @attribute({
        label: t`Enable Route Cache`,
        validation: yup.boolean()
    })
    route_cache_enabled: boolean = false;

    @attribute({
        label: t`Enable Loop Clustering`,
        validation: yup.boolean()
    })
    loop_clustering_enabled: boolean = false;

    @attribute({
        label: t`Enable Job Create Workstation`,
        validation: yup.boolean()
    })
    job_create_workstation_enabled: boolean = false;

    @attribute({
        label: t`Enable Rework?`,
        validation: yup.boolean()
    })
    job_rework_enabled: boolean = false;

    @attribute({
        label: t`Max Allowed Delayed Jobs`,
        validation: yup.number()
    })
    max_allowed_delayed_jobs: number = 2000;

    @computed get use_inf(): boolean {
        return this.inf_enabled;
    }

    @computed
    get isMESLevelRouting(): boolean {
        return this.routing_level.toLowerCase() === 'mes';
    }

    @computed
    get isUsingOPCUA(): boolean {
        return this.connection_type === 'OPCUA';
    }

    @computed
    get tagSourceIsRxData(): boolean {
        return this.tag_data_source === 'rx-data'
    }

    @computed
    get tagSourceIsLms(): boolean {
        return this.tag_data_source === 'lms'
    }

    constructor() {
        super();
        super.setup();
    }
}

export const SettingStore = SettingModel.store as DataStore<SettingModel>;
