class UpdateJobStateJob < ApplicationJob
  #Can be queued as machine_job_create since we need a sequential job when creating a new job.
  queue_as :default

  def perform(job_params, machine_params, job_state_params, re_queued=false)
    msg_base = "Received job state #{job_state_params[:state]} | job: #{job_params[:number]} (side: #{job_state_params[:side]}) | sn: #{machine_params[:sn]}."

    machine = Machine.find_by(sn: machine_params[:sn])
    return Rails.logger.info "#{msg_base} - No machine match that sn." if machine.nil?

    job_process = machine.job_process
    return Rails.logger.warn "#{msg_base} - Machine has no process." if job_process.nil?

    # First try to find the job by job number, if we can't find it check it's aliases.
    # Note: even if the aliases contain the job number, in some scenario we won't have the job number aliases
    # right away (for example, machine start where both job states come in quick succession)
    job = Job.find_by_number(job_params[:number])
    if job.nil?
      job = Job.where("job_numbers LIKE '%\""+job_params[:number]+"\"%'").first
    end

    if job.nil?
      if machine.allow_job_start
        return UpdateJobStateJob.set(queue: :machine_job_create).perform_later(job_params, machine_params, job_state_params, true) unless re_queued
        #TODO: issue - this doesn't work when the two lens have different numbers, see http://omics.dyndns.biz:4747/rx-suite/modules/optional/routing/router/-/issues/314
        return new_job_with_stashed_processes(job_params[:number], machine, job_state_params[:state], job_state_params[:side], job_state_params[:time])
      else
        return Rails.logger.info "#{msg_base} - No job matches this number, ignoring data."
      end
    end

    if machine.restart_jobs?
      #Job exists and has processes that have states which are further down the line than this one
      current_rank = machine.process_rank
      if job.process_changes_further_than_rank? current_rank
        return UpdateJobStateJob.set(queue: :machine_job_create).perform_later(job_params, machine_params, job_state_params, true) unless re_queued
        Rails.logger.info "#{msg_base} - Restarting Job #{job.number} because some of its processes had been started"
        return new_job_with_stashed_processes(job_params[:number], machine, job_state_params[:state], job_state_params[:side], job_state_params[:time])
      end
    end

    if job.is_waiting_on_req_processes?
      Rails.logger.info "#{msg_base} - Job is waiting on required processes. Stashing state change."
      return StashedProcessChange.create(job: job, machine: machine, state: job_state_params[:state], side: job_state_params[:side], time: job_state_params[:time])
    end

    required_process = job.required_processes.find_by(job_process_id: job_process.id, side: job_state_params[:side])

    return Rails.logger.info "#{msg_base} - Job does not require this process for specified side." if required_process.nil?

    Rails.logger.info "#{msg_base}"

    job_state_comment = RequiredProcess.state_history_comment job_state_params
    machine_name = machine_params[:name]
    if job_state_params[:substate] == 'AUTOFINISHED'
      machine_name = (machine.component[:name] || '') + ' (AUTO)'
    end

    # While the router is on, it will broadcast the process data, so don't need to do anything other than save the history
    if JobRouter::Client.in_ready? == false || job.is_routing == false
      if RequiredProcess.job_state_is_stale(job_state_params)
        job.set_reject(Job.reject_data[:timeSyncError], "Job received #{job_process.name} - #{job_state_params[:state].upcase} from #{machine.component.name} with a time difference of over an hour in the past. Please verify machine time and timezone settings.") unless job.is_rejected?
        required_process.required_process_state_histories.new(state: job_state_params[:state].upcase, source: machine_name, job_state_timestamp: job_state_params[:time], comment: job_state_comment)
        required_process.save
        WorkerBroadcast.broadcast_model(job, WorkerBroadcast.action[:update])
      else
        if required_process.opposite_is_rework?
          if required_process.finished?
            Rails.logger.info "#{msg_base} - The opposite required process is in rework mode, ignoring state change"
            # If the process is finish, just add the new data to the history and update the time stamp
            required_process.required_process_state_histories.build(state: job_state_params[:state], source: machine_name, job_state_timestamp: job_state_params[:time], comment: job_state_comment)
            required_process.set_state(required_process.state, required_process.last_location_received_at, job_state_params[:time].in_time_zone(ENV['TZ'])+1.seconds, required_process.last_comment, false)
          else # if the process is not finished, update it as normal
            required_process.set_state(job_state_params[:state], machine_name, job_state_params[:time], job_state_comment)
          end
        else
            required_process.set_state(job_state_params[:state], machine_name, job_state_params[:time], job_state_comment)
        end
        WorkerBroadcast.broadcast_model(required_process, WorkerBroadcast.action[:update])
      end
    # If the job state was created before the job exists, then the router would have ignore it.
    # This for cases where a machine starts a job. If it sends more than one state in quick succession, only the first one is registered.
    elsif job_state_created_before_job?(job_state_params, job)
      Rails.logger.info "#{msg_base} - state created before job, assuming router did not get the state."
      required_process.set_state(job_state_params[:state], machine_name, job_state_params[:time], job_state_comment)
      JobRouter::Client.job_update(job)
      WorkerBroadcast.broadcast_model(required_process, WorkerBroadcast.action[:update])
    else
      required_process.required_process_state_histories.new(state: job_state_params[:state].upcase, source: machine_name, job_state_timestamp: job_state_params[:time], comment: job_state_comment)
      required_process.save
      WorkerBroadcast.broadcast_model(required_process, WorkerBroadcast.action[:update])
    end
  end

  def job_state_created_before_job?(job_state, job)
    job_state[:time] < job.created_at
  end

  def new_job_with_stashed_processes(job_number, machine, state, side, time)
    stashed_proc = StashedProcessChange.create(machine: machine, state: state, side: side, time: time)
    job = JobRequest.data_for_job_number(job_number, 'new', [stashed_proc])
    Rails.logger.info "Creating Job #{job_number} with stashed processes"
    JobRouter::Client.job_create(job)
  end
end