class JobProcessCalculator
  include Singleton

  class JobProcessCalculatorCacheInvalid < StandardError; end

  def initialize
    @build_cache_lock = Mutex.new
  end

  # Returns the jobs processes for a job that has no tag
  def default_processes
    get_required_processes_from_tags({left: {}, right: {}}, [:right, :left])
  end

  def get_required_processes_from_tags(job_tag_data, requested_sides)
    ensure_cache

    return [] if requested_sides.length == 0

    required_processes = []

    # Convert job tag data to JSON to be applied directly with JSON logic
    job_tag_data[:left] = job_tag_data[:left].as_json unless job_tag_data[:left].nil?
    job_tag_data[:right] = job_tag_data[:right].as_json unless job_tag_data[:right].nil?

    @processes.each do|process_data|
      requested_sides.each do |side|
        unless process_data[:condition].nil?
          next unless tags_satisfy_condition(process_data[:condition], job_tag_data[side])
        end
        required_process = build_required_process_definition(process_data, side)

        potential_capabilities = []

        process_data[:capabilities].each do |capability_data|
          unless capability_data[:condition].nil?
            if process_data[:capability_resolve_mode] == "highest_rank"
              # If capability resolve mode is :highest_rank, check the condition on both sides, if either side satisfies the condition, add the capability
              next unless requested_sides.any? do |job_side|
                tags_satisfy_condition(capability_data[:condition], job_tag_data[job_side])
              end
            else
              # If capability resolve mode is :require_all, check the individual side for capability, only the side itself can satisfy the condition
              next unless tags_satisfy_condition(capability_data[:condition], job_tag_data[side])
            end
          end
          potential_capabilities << {name: capability_data[:name], rank: capability_data[:rank]}
        end

        #Set capabilities according to mode capability resolve mode
        if process_data[:capability_resolve_mode] == "highest_rank"
          # Remove any capability with no rank
          potential_capabilities.filter!{|c| c[:rank]}
          # Select lowest rank (rank 1 is best)
          required_process[:capabilities] = if potential_capabilities.length == 0
                                              []
                                            else
                                              required_process[:capabilities] = [potential_capabilities.sort! { |a,b| a[:rank] <=> b[:rank]}[0][:name]]
                                            end
        else
          required_process[:capabilities] = potential_capabilities.map{ |c| c[:name]}
        end

        required_processes << required_process
      end
    end

    required_processes
  end

  def cache_is_built?
    !(@tags.nil? || @processes.nil?)
  end

  def clear_cache!
    log "Flushing data cache."
    @tags = nil
    @processes = nil
  end

  private

  def ensure_cache
    # Check if the cache is built now so we don't have to checkout the mutex constantly
    return if cache_is_built?

    log "Cache requires building, attempting to acquire mutex."

    @build_cache_lock.synchronize do
      # If another thread built the cache, abort now
      return if cache_is_built?

      clear_cache!
      build_cache!
    end
  end

  def build_cache!
    log "Building data cache."
    build_tags
    build_processes
  end

  def build_tags
    @tags = {}
    Tag.pluck(:tag, :value_type, :default_value).each do |tag_data|
      @tags[tag_data[0]] = {
        type: tag_data[1],
        default: tag_data[2]
      }
    end
  end

  def build_processes
    @processes = []
    JobProcess.includes(:tag_condition).where(is_control_process: false).pluck(:id, :name, :rank, :condition, :capability_resolve_mode).each do |process_data|
      data = {name: process_data[1], rank: process_data[2], condition: process_data[3].nil? ? nil : JSON.parse(process_data[3]), capabilities: [], capability_resolve_mode: process_data[4]}

      data[:capabilities] = Capability.includes(:tag_condition).where(job_process_id: process_data[0]).pluck(:name, :condition, :rank).map do |capability_data|
        {name: capability_data[0], condition: capability_data[1].nil? ? nil : JSON.parse(capability_data[1]), rank: capability_data[2] }
      end

      # Only sort capabilities if they have a rank (capabilities only have a rank if 'choose highest' mode)
      unless data[:capabilities].any? {|c| c[:rank].nil?}
        data[:capabilities].sort! { |a,b| a[:rank] <=> b[:rank]}
      end

      @processes << data
    end
    @processes.sort! { |a,b| a[:rank] <=> b[:rank]}
  end

  def tags_satisfy_condition(condition, side_tags)
    target_tags = condition['data']
    job_resolved_tags = {}

    return true if target_tags.blank?
    return false unless side_tags.respond_to?("key")

    # Note do not assign directly to the target_tags as it's shared between all instances of calculations
    target_tags.each_key do |tag_name|
      tag_value = nil
      if side_tags.key?(tag_name)
        tag_value = side_tags[tag_name]
      end

      tag = @tags[tag_name]

      if tag.nil?
        Rails.logger.warn "[Process Calculator] A condition is using the tag #{tag_name} but no such tag is defined. Process calculation may be inaccurate."
        return false
      end

      # If the tag has no value, we may need to set one
      tag_value = tag_default_value(tag) if tag_value.nil?
      tag_value = convert_to_number tag_value unless tag_string_type(tag[:type])

      job_resolved_tags[tag_name] = tag_value
    end

    # Important note: some operations may return unexpected results if a tag has a value of nil.
    # In those instances, the customer site should configure default values to the tag.
    # Some examples:
    #  nil > 0 => false
    #  nil > -1 => true
    return JSONLogic.apply(condition['logic'], job_resolved_tags)
  end

  def tag_default_value(tag)
    if tag[:default]
      return tag[:default]
    elsif tag_string_type(tag[:type])
      return ""
    else
      nil
    end
  end

  def tag_string_type(type)
    return type == "string"
  end

  def build_required_process_definition(process_data, side)
      {
        name: process_data[:name],
        capabilities: [],
        rank: process_data[:rank],
        state:  RequiredProcess.notstarted,
        side: side
      }
  end

  def convert_to_number(value)
    return value if value.nil?
    # If the tag value is already a number, exit now
    return value if value.is_a?(Integer) || value.is_a?(Float)

    # The LMS sometimes sends us numbers as strings for some reason
    tmp = value.tr('"', '')

    return Integer tmp if is_int? tmp

    if is_float? tmp
      tmp = Float tmp

      # Attempt to type cast the float to an int if there is no decimal (so conditions such as 1.0 == 1 don't fail)
      tmp = tmp.to_i if tmp % 1 == 0

      return tmp
    end

    log "[Process Calculator] Failed to convert value #{value} to a number. Process calculation may be inaccurate."

    value
  end

  def is_int?(value)
    Integer(value) && true rescue false
  end

  def is_float?(value)
    Float(value) && true rescue false
  end

  def log(msg)
    Rails.logger.info "[Job Process Calculator] #{msg} "
  end
end
