module JobRouter
  class Client
    class RouterClientNotConnected < StandardError; end

    require 'json'

    def self.info
      JobRouterControl.first
    end

    def self.broadcast_startup_state(message, step)
      max_step = 11.0
      progress = (step / max_step) * 100

      ActionCable.server.broadcast "router", {
          last_broadcast: "#{message}...",
          startup_progress: progress.floor
      }
    end

    def self.ping_client
      Rails::logger.info "Sending ping router server"
      begin
        resp = RxModule::Communicator.call_module_method('router_api', '/status', :get)
        set_router_state(build_state(resp[:state][:status], resp[:state][:message]))
      rescue
        set_router_state(uninitialized)
      end
    end

    def self.set_router_state(status_data)
      router_info = info
      if status_data['status'] != router_info.status
        router_info.update(status: status_data['status'], last_server_message: status_data['message'])
      end
    end

    def self.build_state(status, message)
      JSON.parse({status: status, message: message}.to_json)
    end

    def self.uninitialized
      build_state('offline', 'connection with client not initialized.')
    end

    def self.job_create(job)
      send_info('/job/create', :post, job.full_client_payload)
    end

    def self.job_delete(job)
      send_info("/job/delete?jobNumber=#{job.number}", :delete)
    end

    def self.job_update(job)
      send_info('/job/update', :put, job.full_client_payload)
    end

    def self.job_set_processes(job)
      send_info('/job/update_process', :put, job.processes_client_payload)
    end

    def self.job_set_reject(job)
      send_info('/job/update_reject', :put, job.reject_client_payload)
    end

    def self.machine_update(machine)
      send_info('/machine/update', :put, machine.update_client_payload)
    end

    def self.device_update(device)
      send_info('/device/update', :put, device.update_client_payload)
    end

    def self.device_force_reset(device)
      send_info('/device/reset', :put, {device: device.component.name})
    end

    def self.connectors_update(connectors)
      send_info('/device/update_connectors', :put, {
        device: connectors[0].component.name,
        connectors: connectors.map do |connector|
          connector.update_client_payload
        end
      })
    end

    def self.opcua_constant_update(device_timer)
      send_info('/opcua/update_constants', :put, device_timer.update_client_payload)
    end

    def self.send_info(endpoint, method, payload=nil)
      return Rails::logger.info "[Router Request] Router is offline, will not send request to #{endpoint}" if JobRouter::Client.is_offline?
      start_time = Time.now
      RxModule::Communicator.call_module_method('router_api', endpoint, method, payload ? package_payload(payload) : nil)
      end_time = Time.now
      Rails::logger.info "[Router Request] Request to endpoint #{endpoint} took #{((end_time - start_time)*1000).round(0)}ms to complete"
    end

    def self.package_payload(payload)
      {
        data: payload,
        timestamp: (Time.now.to_f * 1000).to_i
      }
    end

    def self.connected?
      return true if info.status != 'offline'
      false
    end

    def self.in_startup?
      info.status == 'startup'
    end

    def self.in_ready?
      info.status == 'ready'
    end

    def self.is_offline?
      info.status == 'offline'
    end

    def self.in_standby?
      info.status == 'standby'
    end

    def self.timestamp_now_ms
      (DateTime.now.to_f * 1000).to_i
    end

    def self.client_error(data)
      message = data['message']
      router_info = JobRouterControl.first
      router_info.last_server_message = message
      router_info.save
      return self.block_error(data) if data[:level] == 'route_block'
      Rails::logger.info "An error occurred: #{message}"
      Error.create(name: message, location: nil)
    end

    def self.block_error(data)
      # Seperated from error in case we want to do something specific
      message = data['message']
      location = data.has_key?('location') ? data['location'] : nil
      Error.create(name: message, location: location)
      Rails::logger.info "An error occurred: #{message}"
    end

    def self.init(i)
      broadcast_startup_state('Initializing Router Client', 1)
      @node_manager_at_exit = @node_manager_at_exit | false
      if @client.nil? || !@client.open?
        if ENV['RAILS_ENV'] == 'production'
          NodeManager.start
          unless @node_manager_at_exit
            Kernel.at_exit do
              router_info = JobRouterControl.first
              router_info.status = 'offline'
              router_info.save
              send_stop_connection
            end
            @node_manager_at_exit = true
          end
        end
      else
        Rails.logger.info "Trying to open client when it's already open?"
        if i < 1
          Rails.logger.info "Closing and trying to open again"
          begin
            @client.close
          rescue => err
            Rails.logger.info "@client is null: #{err}"
          end
          @client = nil
          init i+1
        else
          Rails.logger.info "Too many attempts, reset router"
        end
      end
    end

    def self.send_stop_connection
      release_routing_jobs
      Rails::logger.info "Sending message stop_connection to client"
      begin
        RxModule::Communicator.call_module_method('router_api', '/terminate', :post)
      rescue => err
        Rails::logger.info "Failed to close connection with client: #{err}"
        return false
      end
      Rails::logger.info "Terminating thread"
      NodeManager.stop
      router_info = JobRouter::Client.info
      router_info.status = 'offline'
      router_info.save
      @node_manager_at_exit = false
      @client = nil
      return true
    end

    def self.release_routing_jobs
      Rails.logger.info 'Releasing all routing jobs...'
      #Note: we do it via update_all to be quick and not trigger a bunch of callbacks. On the front-end we'll force the job store to refresh
      Job.update_all(is_routing: false)
    end
  end
end
