class InfRequest < ApplicationRecord
  belongs_to :job, required: true

  def self.inf_for_job(job)
    Rails::logger.info "Creating inf request for job #{job.number}"

    job.create_inf_request
  end

  def complete
    job.update(inf_target_machine: nil)
    WorkerBroadcast.broadcast_model(job, WorkerBroadcast.action[:update])
    JobRouter::Client.job_update(job)
    self.destroy!
  end

  def can_retry?
    attempt < Setting.first.inf_max_attempts
  end

  def attempt_inf
    machine = Machine.where(id: job.inf_target_machine).first

    return invalid_inf_request(self, "Machine #{job.inf_target_machine} does not exist.") if machine.nil?
    return invalid_inf_request(self, "Machine #{machine.component.name} is not configured to use Job INF.") if machine.lms_config.nil?
    return invalid_inf_request(self, "Machine #{machine.component.name} does not have a process.") if machine.job_process.nil?

    process_inf_tag = machine.job_process.inf_target_tag
    return invalid_inf_request(self, "Process #{machine.job_process.name} is not assigned an INF tag.") if process_inf_tag.blank?

    required_processes = job.required_processes.where(job_process_id: machine.job_process.id)

    if required_processes.blank?
      Rails::logger.info "[INF Request - Job #{request.job.number}] Process #{machine.job_process.name} is no longer required."
      return
    end

    # Validation is ok - attempt the INF call

    Rails::logger.info "[INF Request - Job #{job.number}] Fetching - Attempt #{attempt + 1}/#{Setting.first.inf_max_attempts}"

    begin
      inf_request = ProcessRequests::Inf.new(job.number, machine.lms_config)
    rescue => err
      Rails::logger.error "#{err}"
      return invalid_inf_request(self, 'Something went wrong while fetching the job INF data.')
    end

    # Any status but 0 is a job abort
    return reject_job(self, :fetchJobInfFailed,"INF request returned is missing job status.") if inf_request.tags['STATUS'].nil?

    # Process does not have INF data yet, try again later if we can, otherwise let it fail as if the process was not completed
    return false if inf_request.inf_process_data[process_inf_tag].nil? && self.can_retry?

    # If we get a status different than 0 or still have no process data, abort both lens
    abort_job = inf_request.inf_process_data[process_inf_tag].nil? || inf_request.tags['STATUS'][0] != '0'

    # Use the INF data to determine each eye state
    required_processes.each do |req_process|
      side = req_process.side == 'left' ? 'L' : 'R'

      if abort_job || inf_request.inf_process_data[process_inf_tag][side].nil?
        req_process.apply_state(RequiredProcess.aborted, machine.component.name, RequiredProcess.process_time_now, 'Lens not processed by equipment.')
        unless job.is_rejected?
          job.set_reject(Job.reject_data[:processAborted], "Process #{req_process.job_process.name} was aborted by #{machine.component.name}.")
        end
      else
        req_process.apply_state(RequiredProcess.finished, machine.component.name, inf_timestamp(inf_request.inf_process_data[process_inf_tag][side]))
      end
      req_process.post_current_as_job_state(machine)
    end

    # The INF request is completed
    true
  end

  private

  def invalid_inf_request(request, reason)
    Rails::logger.warn "[INF Request - Job #{request.job.number}] Unable to complete: #{reason}"
    reject_job(request, :fetchJobInfFailed, reason)
  end

  def reject_job(request, reason_key, message)
    Rails::logger.info "[INF Request - Job #{request.job.number}] Rejecting Job - #{message}"
    request.job.set_reject(Job.reject_data[reason_key], message)
    true # If the job is rejected, the INF request is completed
  end

  def inf_timestamp(side_inf_data)
    timestamp = side_inf_data["timestamp"].to_s
    return nil if timestamp.nil?
    timestamp.in_time_zone(ENV['TZ']).to_i
  end

  def self.abort(job_number)
    Rails::logger.info "Job request already exists for number #{job_number}. Aborting new request."
  end

  def self.router_inf_request
    "job_inf"
  end
end
