class Job < ApplicationRecord
  include BatchDependentAssociations
  include TimedConnection

  has_one :inf_request, dependent: :destroy
  has_many :required_processes, dependent: :destroy
  has_many :job_histories, dependent: :delete_all
  has_many :scheduled_process_changes, dependent: :delete_all
  has_many :stashed_process_changes, dependent: :delete_all
  belongs_to :inf_target_machine, class_name: 'Machine', required: false

  def is_rejected?
    return false if self.reject_reason.nil?
    true
  end

  def is_stale?
    settings = Setting.first
    if Time.now - self[:updated_at] > settings[:job_stale_time].to_f
      Rails::logger.info "Job #{self[:number]} is stale"
      true
    else
      false
    end
  end

  # When a job undergoes manual changes, certain actions have to be taken (un-reject the job, clear its INF status, etc.)
  def manual_change
    return_value = nil
    history_to_be_added = []
    history_to_be_added << {event_type: 'manual_process_change (before)', state: process_snapshot}
    if block_given?
      return_value = yield
    end
    un_reject if is_rejected?
    self.inf_target_machine = nil
    history_to_be_added << {event_type: 'manual_process_change (after)', state: process_snapshot}
    self.job_histories.build(history_to_be_added)
    self.save
    # TODO: Currently this method can only be used to wrap process changes, but by changing the payload to send the full job, this method will be able to wrap other manual actions such as un-rejecting a job. This does require modifying the router to handle full payloads on all messages.
    JobRouter::Client.job_set_processes(self)
    return_value
  end

  def set_inf(machine)
    update(inf_target_machine: machine)
  end

  def remove_from_inf_queue
    update(inf_target_machine: nil)
  end

  def is_waiting_on_req_processes?
    return false if self.is_rejected?
    self.required_processes.length == 0
  end

  def process_changes_further_than_rank?(rank)
    required_processes.joins(:job_process).where("rank > #{rank}").where.not( "state = '#{RequiredProcess.notstarted}'").exists?
  end

  def update_location(location_data, timestamp)
    history = location_data[:locations].each_with_index.map do |location, i|
      {
        event_type: 'job_update_location',
        location: location[:location],
        connector: location[:connector],
        state: nil,
        active: true,
        time: timestamp ? Time.at(timestamp/1000.0) : nil,
        is_estimated: i == 0 ? false : true
      }
    end
    self.job_histories.build(history)
    if latest_location?(timestamp)
      self.last_update_location_time = timestamp ? Time.at(timestamp/1000.0) : nil
      self.last_known_location = location_data[:last_known_location][:location]
      self.last_known_connector = location_data[:last_known_location][:connector]
      unless location_data[:next_expected_location].nil?
        self.next_expected_location = location_data[:next_expected_location][:location]
        self.next_expected_connector = location_data[:next_expected_location][:connector]
      end
    end
    self.save
  end

  def latest_location?(timestamp)
    return true if self.last_update_location_time.nil?
    Time.at(timestamp/1000.0) > self.last_update_location_time
  end

  def latest_reject?(timestamp)
    return true if self.last_update_reject_time.nil?
    Time.at(timestamp/1000.0) > self.last_update_reject_time
  end

  def set_complete(timestamp)
    self.is_routing = false
    self.job_histories.build(event_type: 'job_complete', location: self.last_known_location, connector: self.last_known_connector, state: nil, active: false, time: Job.get_time_from_timestamp(timestamp), is_estimated: true)
    save!
  end

  #todo: rework to be more like un_reject so it includes an option to broadcast directly (if appropriate)
  def set_reject(reject_data, force_description = nil, timestamp: nil)
    set_reject_data(reject_data[:key], force_description.nil? ? reject_data[:description] : force_description)
    self.job_histories.build(event_type: 'job_set_reject', reject_reason: self.reject_reason, location: self.last_known_location, connector: self.last_known_connector, state: nil, active: true, time: Job.get_time_from_timestamp(timestamp), is_estimated: true)
    save!
  end

  def set_reject_data(reason, description)
    self.reject_reason = reason
    self.reject_description = description
  end

  def un_reject(broadcast_to_router=true, timestamp: nil)
    set_reject_data(nil, nil)
    self.job_histories.new(event_type: 'job_set_unreject', reject_reason: nil, time: Job.get_time_from_timestamp(timestamp))
    self.save!
    if broadcast_to_router
      JobRouter::Client.job_set_reject(self)
    end
  end

  def processes_client_payload
    {
        jobNumber: self.number,
        requiredProcesses: self.format_processes,
        isWaitingOnInf: self.inf_target_machine ? true : false
    }
  end

  def reject_client_payload
    {
        jobNumber: self.number,
        reject: is_rejected?
    }
  end

  def full_client_payload
    json = self.get_json
    # Router no longer needs the starting location of each job
    #last_non_estimated_job_history = self.job_histories.where(is_estimated: false).last
    last_non_estimated_job_history = nil
    if last_non_estimated_job_history.nil? || last_non_estimated_job_history.location.nil?
      json[:location] = nil
    else
      json[:location] = {node: last_non_estimated_job_history[:location], connector: last_non_estimated_job_history[:connector]}
    end
    json[:jobNumberAliases] = self.job_numbers.nil? ? [] : JSON.parse(self.job_numbers)
    json[:isWaitingOnInf] = self.inf_target_machine ? true : false
    json
  end

  def do_delayed_process_change_for_router(job_process, state, machine)
    r_process = self.required_processes.find_by(job_process_id: job_process.id, side: 'right')
    l_process = self.required_processes.find_by(job_process_id: job_process.id, side: 'left')

    unless r_process.nil?
      if machine.is_mes_machine
        # If it's an MES machine, apply the process but don't update the history, instead it to Rx-Data
        r_process.apply_state(state, machine.component.name)
        r_process.post_current_as_job_state(machine)
      else
        r_process.set_state(state, machine.component.name)
      end
    end

    unless l_process.nil?
      if machine.is_mes_machine
        l_process.apply_state(state, machine.component.name)
        l_process.post_current_as_job_state(machine)
      else
        l_process.set_state(state, machine.component.name)
      end
    end

    JobRouter::Client.job_set_processes(self)
  end

  def required_process_started?(job_process)
    r_process = self.required_processes.find_by(job_process_id: job_process.id, side: 'right')
    l_process = self.required_processes.find_by(job_process_id: job_process.id, side: 'left')

    r_started = r_process.nil? ? false : r_process.started?
    l_started = l_process.nil? ? false : l_process.started?

    r_started || l_started
  end

  def synchronize_external_data
    return unless Setting.use_rx_data_jobs?
    if self.rx_data_job_id.nil?
      Rails::logger.info "Job #{self.number} has no Rx-Data Job Id. Skipping Synchronization."
      return
    end

    begin
      resp = RxModule::Communicator.call_module_method('rx_data', "/api/jobs/#{self.rx_data_job_id}/latest_routing_states", :get, nil, nil, timed_connection(30))
      resp[:job_states].each do |job_state|
        # If the machine is not in use for routing, or has no process, ignore the job state
        machine = Machine.find_by(sn: job_state[:machine_sn])
        next if machine.nil? || machine.job_process.nil?

        # If the process is no longer required, ignore the job state
        required_process = self.required_processes.find_by(job_process_id: machine.job_process.id, side: job_state[:side])
        next if required_process.nil?

        timestamp = RequiredProcess.insure_time_validity(job_state[:time]).to_i
        required_process.update_with_external(job_state[:state], timestamp, job_state[:comment], machine.component.name)

        if required_process.state == 'ABORTED'
          unless self.is_rejected?
            self.set_reject(Job.reject_data[:processAborted], "Job process #{required_process.job_process.name} was aborted by a machine.")
          end
        end
      end
    rescue => err
      Rails::logger.warn "Error while synchronizing job #{self.number} (rx-data id: #{self.rx_data_job_id}): #{err}"
      self.set_reject(Job.reject_data[:failedToSynchronize])
    end
  end

  def verify_before_send_to_router
    ensure_reject_validity
    ensure_process_validity
  end

  def broadcast_to_router(action)
    if action == :create
      return JobRouter::Client.job_create(self)
      #TODO: update will merge into :update_data so that the full payload is always sent
    elsif  action == :update
      return JobRouter::Client.job_update(self)
    elsif action == :update_data
      return JobRouter::Client.job_create(self)
    end
    raise NotImplementedError
  end

  def self.post_to_rx_data(number, tags, processes)
    Rails::logger.info "Posting Job Number #{number} to Rx-Data."
    resp = RxModule::Communicator.call_module_method('rx_data', "/api/orders/create_from_routing", :post, {
      mes: {
        evt: {
          id: number,
          order: {
            name: number,
            description: 'Created by Rx-Routing',
            data: Job.tags_for_rx_data(tags),
            processes: Job.processes_for_rx_data(processes)
          }
        }
      }
    })
    resp
  end

  def self.tags_for_rx_data(tags_by_side)
    tag_data = {
      item: []
    }
    tags_by_side.each do |side, tags|
      # Unlike processes, if a tag data is shared by both eyes, still send up separately (cannot use do 'B')
      next if tags.nil?
      tags.each do |tag, val|
        tag_data[:item] << {name: tag, lensside: side.to_s == "left" ? "L": "R", value: val}
      end
    end
    tag_data
  end

  def self.processes_for_rx_data(processes)
    process_data = {
      process: []
    }
    processes.each do |p|
      process = process_data[:process].detect { |v| v[:name] == p[:name]}
      # If the process is shared by both eye, set its side to B, otherwise use the side.
      if process.nil?
        process_data[:process] << {name: p[:name], eye: p[:side] == "left" ? "L": "R"}
      else
        process[:eye] = "B"
      end
    end
    process_data
  end

  def self.create_new(number)
    Job.ensure_does_not_exist(number)
    job = Job.new(number: number, active: true)
    job.job_histories.build(event_type: 'job_create', active: true, is_estimated: true)
    job.save!
    job
  end

  def do_delete
    #because a lot of data is tied to jobs, delete the record directly and let the DB associations take care of the rest
    JobRouter::Client.job_delete(self)
    self.destroy!
  end

  def self.ensure_does_not_exist(number)
    old_job = Job.find_by(number: number)
    unless old_job.nil?
      Rails::logger.info "Job #{number} is already in use. Removing existing job data."
      old_job.destroy!
    end
  end


  def self.reject_data
    {
        receivedStartInReject:      {key: 'receivedStartInReject',      description: 'Job received to start in reject.'},
        failedToSynchronize:        {key: 'failedToSynchronize',        description: 'Rx-Router was unable to fetch the most recent job states for this job.'},
        processAborted:             {key: 'processAborted',             description: 'A process of this job was aborted.'},
        noProcessesAtNonStart:      {key: 'noProcessesAtNonStart',      description: 'Job appeared on the line without visiting a start point.'},
        failFetchProcesses:         {key: 'failFetchProcesses',         description: 'Failed to fetch this job process data.'},
        timeSyncError:              {key: 'timeSyncError',              description: 'Job received a job state that is much older than the current process state.'},
        fetchJobInfFailed:          {key: 'fetchJobInfFailed',          description: 'Something went wrong while fetching the INF data'}
    }
  end

  def finish
    self.required_processes.each do |req_process|
      req_process.set_state(RequiredProcess.finished, RequiredProcess.source_manual)
    end
    job_histories.new(event_type: 'manual_finish')
    JobRouter::Client.job_set_processes(self) if save
  end

  def reset
    self.required_processes.each do |req_process|
      req_process.set_state(RequiredProcess.notstarted, RequiredProcess.source_manual)
    end
    job_histories.new(event_type: 'manual_finish')
    JobRouter::Client.job_set_processes(self) if save
  end

  def process_snapshot
    self.required_processes.all.map do |required_process|
      {
          name: required_process.job_process[:name],
          side: required_process[:side],
          state: required_process[:state],
          force_to_machine: required_process[:force_to_machine],
          capabilities: required_process.capabilities.all do |required_capability|
            {
                name: required_capability[:name]
            }
          end
      }
    end.to_json
  end

  def set_required_processes(req_processes_data)
    # Important note: we set the initial process time to 2 minute from now to solve a problem where job states that
    # start a job sometimes arrive a few seconds after the job was created (which mean they would normally be ignored).
    # While not ideal, it's the least intrusive method to deal with this problem.
    start_time = Time.now - 2.minute
    req_processes_data.each do |req_process_data|
      job_process = JobProcess.find_by_name(req_process_data[:name])
      next if job_process.nil?
      req_process = self.required_processes.new(job_process: job_process, side: req_process_data[:side])
      req_process.capabilities << job_process.capabilities.where(name: req_process_data[:capabilities])
      #TODO NOTE: this triggers a save. The save should be moved (or made optional)
      req_process.set_state(RequiredProcess.notstarted, RequiredProcess.source_manual, start_time)
    end
    self.job_histories.create(event_type: 'process_set', location: self[:location], connector: self[:connector], state: self[:job_state], active: true, is_estimated: true)
  end

  def apply_stashed_processes
    req_process_changes = StashedProcessChange.where(job: self).order(:time)

    req_process_changes.each do |stashed_process|
      req_process = self.required_processes.find_by(job_process: stashed_process.job_process, side: stashed_process.side)

      if req_process.nil?
        Rails::logger.warn "Unable to apply stashed process #{stashed_process.job_process.name} - #{stashed_process.state} to job #{self.number}. Process not required."
        next
      end

      req_process.set_state(stashed_process.state, stashed_process.machine.component.name, stashed_process.time)
    end

    req_process_changes.in_batches.destroy_all unless req_process_changes.empty?
  end

  def self.get_time_from_timestamp(timestamp)
    return Time.now if timestamp.nil?
    Time.at(timestamp/1000.0).in_time_zone(ENV['TZ'])
  end

  def get_json
    require 'json'
    {
        jobNumber: self.number,
        state: self.state,
        location: self.last_known_location.nil? ? nil : {
            node: self.last_known_location,
            connector: self.last_known_connector
        },
        requiredProcesses: format_processes,
        rejectReason: self.reject_reason ? {
            key: self.reject_reason,
            description: self.reject_description
        } : nil
    }
  end

  def format_processes
    process_hash = {}
    self.required_processes.find_each do |required_process|
      unless process_hash.has_key?(required_process.job_process[:name])
        process_hash[required_process.job_process[:name]] = {
            name: required_process.job_process[:name],
            rank: required_process.job_process[:rank],
            forceToMachine: required_process[:force_to_machine],
            lensData: {}
        }
      end
      process_hash[required_process.job_process[:name]][:lensData][required_process[:side]] = {
                                                           side: required_process[:side],
                                                           state: required_process[:state],
                                                           requiredCapabilities: required_process.capabilities.all.map do |capability|
                                                             capability[:name]
                                                           end,
                                                           lastLocationReceivedAt: required_process[:last_location_received_at],
                                                           lastStateChangeTimestamp: required_process[:last_state_change_timestamp],
                                                           lastComment: required_process[:last_comment],
                                                           isRework: required_process.is_rework?
                                                       }
    end
    process_hash.map do |key, process|
      process
    end
  end

  def set_processes_for_side(processes, capabilities, side)
    #TODO: We're only returning the processes to broadcast for the moment to support the weird logic in jobs_controller.rb update_required_processes, once we fix that, we should also remove this
    processes_to_broadcast = []

    processes.each do |process_name, required|
      process = JobProcess.find_by(name: process_name)
      required_process = self.required_processes.find_by(job_process_id: process.id, side: side)

      if required
        if required_process.nil?
          required_process = RequiredProcess.create(side: side, job_process: process)
          self.required_processes << required_process
          required_process.set_state(RequiredProcess.notstarted, RequiredProcess.source_manual)
        end
        required_process.set_capabilities_from_ids(capabilities[process_name])
        processes_to_broadcast.push(required_process)
      else
        required_process.destroy unless required_process.nil?
      end
    end

    processes_to_broadcast
  end

  after_create do |model|
    ActionCable.server.broadcast "everything", {
        model: JSON.parse(render_jbuilder_partial),
        class: model.class.name,
        action: :create
    }
  end

  after_update do |model|
    ActionCable.server.broadcast "everything", {
        model: JSON.parse(render_jbuilder_partial),
        class: model.class.name,
        action: :update
    }
  end

  after_destroy do |model|
    ActionCable.server.broadcast "everything", {
        model: JSON.parse(render_jbuilder_partial),
        class: model.class.name,
        action: :destroy
    }
  end

  def should_broadcast?
    false
  end

  def self.release_routing_jobs
    Job.update_all(is_routing: false)
  end

  def self.job_message_target(data, is_new=false)
    number = extract_job_number(data)
    job = Job.find_by_number(number)
    if job.nil? && !is_new
      Rails::logger.info "Found no job matching number #{number}"
    end
    # Check if the timestamp is before the job created_at time,
    # if so this update is probably for a previous job with the same number
    return nil if job && Job.get_time_from_timestamp(data[:timestamp]) < job.created_at
    job
  end

  def self.extract_job_data(data)
    {
        job_number: extract_job_number(data),
        locations: extract_job_location_data(data)
    }
  end

  def self.extract_job_location_data(data)
    {
      last_known_location: {
        location: data['currentLocation']['node'],
        connector: data['currentLocation']['connector']
      },
      next_expected_location: data['nextLocation'].nil? ? nil : {
        location: data['nextLocation']['node'],
        connector: data['nextLocation']['connector']
      },
      locations: data['locations'].map do |location_hash|
        {
          location: location_hash['location']['node'],
          connector: location_hash['location']['connector'],
          timestamp: location_hash['timestamp']
        }
      end
    }
  end

  def self.extract_job_reject_data(data)
    unless data['isReject']
      return nil
    end
    {
        reason: data['rejectData']['key'],
        description: data['rejectData']['description']
    }
  end

  def self.extract_job_process_data(data)
    {
        state:  data['state'],
        last_location_received_at: data['lastLocationReceivedAt'],
        last_state_change_timestamp: data['lastStateChangeTimestamp'],
        last_comment: data['lastComment']
    }
  end

  def self.extract_job_number(data)
    data["jobNumber"]
  end

  private

  # Some reject reasons are 'temporary' and should be removed when the job is being re-fetched
  def ensure_reject_validity
    return unless is_rejected?

    if self.reject_reason === Job.reject_data[:failedToSynchronize][:key]
      un_reject(false)
    end

    #TODO: this might be a good place to un-reject a job that was rejected for process abort that is no longer in abort.

  end

  # Remove any process that are no longer obtainable on the routing graph
  def ensure_process_validity
    process_ids = RoutingGraph.first.process_ids_offered
    self.required_processes.where.not(job_process_id: process_ids).in_batches.destroy_all
  end
end