class LmsConfig < ApplicationRecord
  class LMSRuleModeIsUnknown < StandardError; end
  class JobNumberDoesntFitAnyLMSConfigRule < StandardError; end
  class InvalidRxLinkResponse < StandardError; end

  has_many :machines
  enum lms_type: [:job, :inf]

  class << self
    def use_lms_priority
      Setting.instance.lms_rule_mode == "priority_based"
    end

    def use_job_number_length
      Setting.instance.lms_rule_mode == "length_based"
    end

    def all_job_lms
      LmsConfig.where(lms_type: 'job')
    end

    def all_job_lms_online?
      all_job_lms.all? do |config|
        config.online
      end
    end

    def ping_all_job_lms
      LmsConfig.all_job_lms.each do |config|
        config.ping
      end
    end

    def find_lms_configs_for_job_number(job_number)
      job_lms_configs = LmsConfig.where(lms_type: :job)
      return job_lms_configs.order(:priority) if use_lms_priority

      if use_job_number_length
        # Get all LMS that:
        # 1- have a lower bound (or equal) than the job number's length
        # 2- have a lower bound (or equal) job number's length AND a upper bound greater than (or equal)
        # 3- have no bounds
        length = job_number.length
        len_configs = job_lms_configs.where(
          arel_table[:length_left_bound].lteq(length).and(arel_table[:length_right_bound].eq(nil))
                                        .or(arel_table[:length_left_bound].lteq(length).and(arel_table[:length_right_bound].gteq(length)))
                                        .or(arel_table[:length_left_bound].eq(nil).and(arel_table[:length_right_bound].eq(nil)))
        ).order(:length_left_bound).reverse_order
        raise JobNumberDoesntFitAnyLMSConfigRule if len_configs.blank?
        return len_configs
      end

      raise LMSRuleModeIsUnknown
    end

    def migrate_legacy_connections
      setting = Setting.instance

      LmsConfig.where(external_dcs_id: nil).find_each do |conf|
        resp = RxModule::Communicator.call_module_method_as_module("rx_link", "/api/dcs_configs", :post, {
          dcs_config: {
            name: conf.name.nil? ? "#{conf.ip}" : conf.name,
            description: 'routing',
            ip: conf.ip,
            port: conf.port,
            device_type: conf.device_type.nil? ? 'DNL' : conf.device_type,
            retry_on_fail_connection: setting.rx_data_max_fetch_attempts, # Was previously for both LMS and Rx-Data
            connection_timeout: setting.rx_data_fetch_timeout, # Was previously for both LMS and Rx-Data
            max_open_connections: conf.max_open_connections <= 0 ? nil : conf.max_open_connections,
            delay_on_socket_open: conf.delay_on_socket_open <= 0 ? nil : conf.delay_on_socket_open,
            hold_connection_open: conf.hold_connection,
            keep_call_trace: false
          }
        })

        raise InvalidRxLinkResponse if resp[:id].blank?

        puts "Successfully registered DCS #{conf.ip}:#{conf.port} with Rx-Link"

        conf.update!(name: nil, ip: nil, port: nil, device_type: nil, delay_on_socket_open: nil, max_open_connections: nil, hold_connection: nil, external_dcs_id: resp[:id])
      rescue => err
        puts "Failed to register DCS #{conf.ip}:#{conf.port} with Rx-Link: #{err}"
      end
    end
  end

  def external_url
    "/api/dcs_configs/#{external_dcs_id}"
  end

  def ping
    return false if external_dcs_id.nil?
    success = RxModule::Communicator.call_module_method_as_module("rx_link", "#{external_url}/ping", :get)[:success]
    update(online: success)
  rescue
    update(online: false)
  end

  def reinitialize
    success = RxModule::Communicator.call_module_method_as_module("rx_link", "#{external_url}/force_reset", :get)[:success]
    update(online: success)
  rescue
    update(online: false)
  end
end
