class Machine < ApplicationRecord
  class MachineDoesNotExistInRxData < StandardError; end

  belongs_to :component, dependent: :destroy, required: true
  belongs_to :job_process, optional: true
  belongs_to :lms_config, required: false
  has_many :capability_machines, :dependent => :destroy
  has_many :capabilities, :through => :capability_machines
  has_many :scheduled_process_changes, dependent: :destroy
  has_many :stashed_process_changes, dependent: :destroy
  has_many :inf_jobs, class_name: 'Job'

  validates_associated :component, message: "'s name is already taken"

  def is_mes_machine
    return false if self.sn.nil?
    true
  end

  def is_sending_job_states?
    sn? && apply_process_change_on == 'use_job_state'
  end

  def machine_state
    if component.connectors.any? {|connector| connector.state == 'online'}
      'online'
    elsif component.connectors.any? {|connector| connector.state == 'standby'}
      'standby'
    else
      'offline'
    end
  end

  def update_client_payload
    if self.sn.nil?
      {
          machine: self.component[:name],
          forcedState:  self.component.connectors.first.state,
          capabilities: self.capability_machines.where(enabled: true).map {|capability_machine| capability_machine.capability.name}
      }
    else
      {
          machine: self.component.name,
          forcedState: self.forced_state,
          onlineStates: self.mes_machine_states.where(online: true).map do |state|
            state[:name]
          end,
          applyProcessChangeOn: self.apply_process_change_on,
          capabilities: self.capability_machines.where(enabled: true).map {|capability_machine| capability_machine.capability.name}
      }
    end
  end

  def inf_enabled?
    !!lms_config & Setting.inf_enabled?
  end

  def restart_jobs?
    restart_jobs
  end

  def process_rank
    job_process.rank
  end

  # Returns this machine and all others that belong to the same rx-data machine group.
  def machines_in_same_group
    return [self] unless is_mes_machine

    machines = Machine.fetch_rx_data_machines
    this_machine = machines.detect {|machine| machine[:sn] == self.sn}

    raise MachineDoesNotExistInRxData if this_machine.nil?

    return [self] if this_machine[:machine_group_ids].nil?

    machines_in_group = machines.select {|machine| !(machine[:machine_group_ids] & this_machine[:machine_group_ids]).empty?}

    machines = machines_in_group.map do |machine_in_group|
      MesMachine.find_by_sn(machine_in_group[:sn])
    end

    machines << self
  end

  def update_from_params(params)
    # Settings that cannot be changed while the router is on
    if JobRouter::Client.is_offline?
      self.force_duration = params[:force_duration]
      self.selected_reject = params[:selected_reject]
      self.allow_job_start = params[:allow_job_start]
      self.restart_jobs = params[:restart_jobs]
      self.lms_config_id = params[:lms_config_id]

      # If the machine process changed, delete all capabilities
      if self.job_process != JobProcess.find(params[:job_process_id])
        self.job_process = JobProcess.find(params[:job_process_id])
        self.capability_machines.delete_all
      end

      self.ignore_state = params[:ignore_state] #TODO: likely not used (2021/08/11)

      # Add new capabilities and delete the ones that are no longer in use
      capabilities_to_delete = self.capability_machines.map {|cap| cap.capability_id}
      requested_capabilities = params[:capability_ids].blank? ? [] : params[:capability_ids]

      requested_capabilities.each do |capability_id|
        if capabilities_to_delete.find {|c| c == capability_id}
          capabilities_to_delete.delete(capability_id)
        else
          capability = Capability.where(id: capability_id).first
          # Only add the capability if the process is the same that's on the machine
          self.capabilities << capability if capability.job_process == self.job_process
        end
      end

      capability_machines.where(capability_id: capabilities_to_delete).delete_all
    end

    self.apply_process_change_on = params[:apply_process_change_on]

    JobRouter::Client.machine_update(self) if save
  end

  def self.fetch_rx_data_machines
    RxModule::Communicator.call_module_method('rx_data', 'api/machines/status_all', :get)
  end

  def self.update_rx_data_machines
    if Setting.use_rx_data_machines?
      begin
        mes_machines = fetch_rx_data_machines.select {|k ,v| v[:is_conveyor_machine]}
      rescue
        Rails.logger.warn "Rx-data Cannot be reached"
        return
        #TODO:Should let user know rx-data offline
      end

      mes_machines.each do |sn, mes_machine|
        next if sn.blank?

        machine = MesMachine.find_by_sn(sn)
        if machine.nil?
          create_from_rx_data(mes_machine.merge!(sn: sn))
        else
          machine.component.update!(name: mes_machine[:name])
          machine.update!(last_reported_state: mes_machine[:status] || 'WAITINGFORJOBS')
        end
      end
    end
  end

  def self.create_from_rx_data(mes_machine)
    component = MesComponent.new(name: mes_machine[:name])
    machine = MesMachine.new(rx_data_id: mes_machine[:id], sn: mes_machine[:sn], forced_state: 'none', last_reported_state: mes_machine[:status] || 'WAITINGFORJOBS')
    machine.component = component
    machine_connectors = [
      {
        name: 'input',
        connector_type: 'input',
        state: 'online'
      },
      {
        name: 'output',
        connector_type: 'output',
        state: 'online'
      }
    ]
    machine_connectors.each do |machine_connector|
      connector = MesConnector.new(machine_connector)
      machine.component.connectors << connector
    end

    machine.mes_machine_states << MesMachineState.new(name: 'WAITINGFORJOBS', online: true)
    machine.mes_machine_states << MesMachineState.new(name: 'PROCESSING', online: true)

    machine.save
    machine
  end

  def self.machine_message_target(data, is_new=false)
    name = data['machine']
    component = Component.find_by_name(data['machine'])
    if component.nil?
      unless is_new
        Rails::logger.info "No machine found with name #{name}"
      end
      return
    end
    component.machine
  end
end