class RequiredProcess < ApplicationRecord
  belongs_to :job
  belongs_to :job_process
  has_many :capability_required_processes, dependent: :delete_all
  has_many :capabilities, :through => :capability_required_processes
  has_many :required_process_state_histories, dependent: :delete_all

  def post_current_as_job_state(machine, as_auto_finished=true)
    return if self.state == RequiredProcess.notstarted # Rx-Data does not recognize the state 'not started'
    PostJobState.perform_later({
                               sn: machine.sn,
                               date: Time.at(self.last_state_change_timestamp).to_date,
                               time: Time.at(self.last_state_change_timestamp).to_time,
                               job_state: {
                                 job: self.job.number,
                                 lensside: self.side == 'left' ? 'L' : 'R',
                                 state: self.state,
                                 comment: self.last_comment,
                                 substate: as_auto_finished ? 'AUTOFINISHED' : nil,
                                 process: self.job_process.name
                               }
                             })
  end

  def update_with_external(state, timestamp, comment=nil, location=nil, do_save=true, rework=nil)
    if self.last_state_change_timestamp && timestamp < self.last_state_change_timestamp
      Rails::logger.info "Tried updating Require process #{self.job_process.name}, side #{self.side} of job #{self.job.number}. The timestamp of the update was older than the current state timestamp"
      return false
    end
    unless rework.nil?
      self.is_rework = rework
    end
    self.state = state.upcase
    self.last_state_change_timestamp = timestamp
    self.last_comment = comment
    self.last_location_received_at = location
    save if do_save
    true
  end

  def set_capabilities_from_ids(capabilities_ids)
    #TODO: this can be optimized by checking which capabilities already exist on process, which one need to be added, which one need to be deleted
    self.capabilities.destroy_all unless self.capabilities.empty?
    capabilities_ids.each do |capability_id|
      capability = Capability.where(id: capability_id)
      self.capabilities << capability
    end
  end

  def set_state(state, source, time=nil, comment=nil, add_to_history=true)
    state = state.upcase
    time = RequiredProcess.insure_time_validity(time)
    timestamp = time.to_i
    self.required_process_state_histories.build(state: state, source: source, job_state_timestamp: time, comment: comment) if add_to_history
    self.update_with_external(state, timestamp, nil, nil, false)
    save
  end

  def apply_state(state, source, timestamp=RequiredProcess.process_time_now, comment='')
    self.update_with_external(state, timestamp, comment, source)
  end

  def opposite_process
    self.job.required_processes.where(job_process_id: self.job_process_id).where(side: self.side == 'left' ? 'right' : 'left').first
  end

  def opposite_process_hash
    opposite = opposite_process
    return nil if opposite.nil?
    opposite_hash = opposite.attributes
    opposite_hash["capability_ids"] = opposite.capability_ids
    opposite_hash
  end

  def opposite_is_rework?
    opp_req_process = opposite_process
    return false if opp_req_process.nil?
    opp_req_process.is_rework?
  end

  def update_opposite_process_front_end
    ActionCable.server.broadcast "everything", {
        model: JSON.parse(opposite_process.render_jbuilder_partial),
        class: self.class.name,
        action: :update
    }
  end

  def started?
    self.state == RequiredProcess.started
  end

  def is_rework?
    self.is_rework
  end

  def finished?
    self.state == RequiredProcess.finished
  end

  def self.insure_time_validity(time)
    if time.nil?
      time = Time.now
    elsif time > Time.now
      time = Time.now
    end
    time.in_time_zone(ENV['TZ'])
  end

  def self.started
    'STARTED'
  end

  def self.finished
    'FINISHED'
  end

  def self.aborted
    'ABORTED'
  end

  def self.notstarted
    'NOTSTARTED'
  end

  def self.rework
    'REWORK'
  end

  def self.source_manual
    'manual'
  end

  def self.source_automatic
    'automatic'
  end

  # The process time must match the local time otherwise job states posted by the machines will not update correctly
  def self.process_time_now
    Time.now.in_time_zone(ENV['TZ']).to_i
  end

  def self.state_history_comment(state_data)
    return nil if state_data[:comment].nil? && state_data[:scrap_reason].nil?
    return state_data[:comment] if state_data[:scrap_reason].nil?
    return state_data[:scrap_reason] if state_data[:comment].nil?
    "#{state_data[:comment]} (scrap: #{state_data[:scrap_reason]})."
  end

  def self.job_state_is_stale(job_state_params)
    time = RequiredProcess.insure_time_validity(job_state_params[:time])
    return (Time.now - time) >= RequiredProcess.stale_data_threshold
  end

  def self.stale_data_threshold
    45.minutes
  end

  after_create do |model|
    ActionCable.server.broadcast "everything", {
        model: JSON.parse(render_jbuilder_partial),
        class: model.class.name,
        action: :create
    }
  end

  after_update do |model|
    ActionCable.server.broadcast "everything", {
      model: JSON.parse(render_jbuilder_partial),
      class: model.class.name,
      action: :update
    }
    #This is necessary to update the required processes buttons in the front-end for them to be disabled
    model.update_opposite_process_front_end unless opposite_process.nil?
  end

  def should_broadcast?
    false
  end
end

