class Setting < ApplicationRecord
  enum lms_rule_mode: [:priority_based, :length_based]

  class << self
    def instance
      Setting.first_or_create!
    end

    def seed(router_level, plc_control, opcua_variant)
      # A lot of these could be done in the setting instantiation, how due to legacy installs we have to seed it
      setting = instance
      router_level = router_level.downcase
      plc_control = plc_control.upcase
      opcua_variant = opcua_variant.downcase unless opcua_variant.nil?

      setting.routing_level = router_level if setting.routing_level.nil? || setting.routing_level != router_level
      setting.connection_type = plc_control if setting.connection_type.nil? || setting.connection_type != plc_control
      setting.variant = opcua_variant if setting.variant.nil? || setting.variant != opcua_variant
      setting.tag_data_source = 'lms' if setting.tag_data_source.nil?
      setting.load_balancing_mode = 'none' if setting.load_balancing_mode.nil?
      setting.use_default_processes_on_fail_fetch = false if setting.use_default_processes_on_fail_fetch.nil?
      setting.mark_job_dead_after_x_visit = -1 if setting.mark_job_dead_after_x_visit.nil?
      setting.use_rx_scanners = plc_control == 'OPCUA' && router_level != 'basic'
      setting.save!

      register_push_targets
    end

    def lms_tag_source?
      instance.tag_data_source.downcase == 'lms'
    end

    def rx_data_tag_source?
      instance.tag_data_source.downcase == 'rx-data'
    end

    def use_opcua?
      instance.connection_type.downcase == 'opcua'
    end

    def use_rx_scanners?
      return false unless use_opcua?
      instance.use_rx_scanners
    end

    def inf_enabled?
      instance.inf_enabled
    end

    def use_rx_data_machines?
      instance.routing_level.downcase == 'mes'
    end

    def use_rx_data_jobs?
      rx_data_tag_source? || instance.post_job_data
    end

    def use_rx_data?
      use_rx_data_jobs? || use_rx_data_machines?
    end

    def configured?
      settings_to_check = [:mark_job_dead_after_x_visit]

      settings_to_check << :opcua_server_url if use_opcua?

      if rx_data_tag_source?
        settings_to_check << :rx_data_max_fetch_attempts
        settings_to_check << :rx_data_fetch_timeout
      end

      return false if settings_to_check.any? {|setting| instance[setting].nil? }

      if lms_tag_source?
        return false if LmsConfig.pluck(:external_dcs_id).any? nil
      end

      true
    end

    def register_push_targets
      begin
        Rails::logger.info  "Registering Rx-Data Push Targets..."
        RxModule::Communicator.call_module_method('rx_data', '/api/register_push_targets', :post, {push_target: {app: 'rx_router', machine: '/api/machines/change_from_external', machine_state: 'api/machines/state_from_external', job_state: '/api/jobs/rx_data_update'}})
        RxModule::Communicator.call_module_method('rx_data', '/api/register_push_targets', :post, {push_target: {app: 'router_api', machine_state: '/mes-data/machine-state', job_state: '/mes-data/job-state' }})
        Rails::logger.info  "Push Targets registered successfully!"
      rescue => err
        Rails::logger.warn "FAILED TO REGISTER PUSH TARGETS: #{err}"
      end
    end
  end

  # Returns the settings for the router to talk with Rx-Scanners directly
  def external_scanner_service_settings
    require 'net/http'

    hub_url = ENV['RX_HUB_URL']
    hub_scanner_endpoint_url = URI.parse("#{hub_url}/api/components/rx_scanners/endpoint")
    req = Net::HTTP::Get.new(hub_scanner_endpoint_url.to_s)
    res = Net::HTTP.start(hub_scanner_endpoint_url.host, hub_scanner_endpoint_url.port) { |http|
      http.request(req)
    }
    res = res.body.tr('/','')
    res = JSON.parse(res)
    res = res['url'].split(':')

    {
        serviceIp: res[1],
        servicePort: res[2],
    }
  end

  after_create do |model|
    ActionCable.server.broadcast "everything", {
        model: JSON.parse(render_jbuilder_partial),
        class: model.class.name,
        action: :create
    }
  end

  after_update do |model|
    ActionCable.server.broadcast "everything", {
        model: JSON.parse(render_jbuilder_partial),
        class: model.class.name,
        action: :update
    }
  end

  after_destroy do |model|
    ActionCable.server.broadcast "everything", {
        model: JSON.parse(render_jbuilder_partial),
        class: model.class.name,
        action: :destroy
    }
  end

  def should_broadcast?
    false
  end
end